# -*- coding: utf-8 -*-

"""Constants for PyBEL tests."""

import logging

from pybel.constants import *
from pybel.dsl import (
    Abundance,
    BiologicalProcess,
    ComplexAbundance,
    CompositeAbundance,
    EnumeratedFusionRange,
    Fragment,
    Gene,
    GeneFusion,
    GeneModification,
    Hgvs,
    HgvsReference,
    HgvsUnspecified,
    MicroRna,
    NamedComplexAbundance,
    Pathology,
    Protein,
    ProteinFusion,
    ProteinModification,
    Reaction,
    Rna,
    RnaFusion,
    secretion,
    translocation,
)
from pybel.dsl.namespaces import hgnc
from pybel.language import activity_mapping, citation_dict, compartment_mapping

logger = logging.getLogger(__name__)

expected_test_simple_metadata = {
    METADATA_NAME: "PyBEL Test Simple",
    METADATA_DESCRIPTION: "Made for testing PyBEL parsing",
    METADATA_VERSION: "1.6.0",
    METADATA_COPYRIGHT: "Copyright (c) Charles Tapley Hoyt. All Rights Reserved.",
    METADATA_AUTHORS: "Charles Tapley Hoyt",
    METADATA_LICENSES: "WTF License",
    METADATA_CONTACT: "cthoyt@gmail.com",
    METADATA_PROJECT: "PyBEL Testing",
}

expected_test_thorough_metadata = {
    METADATA_NAME: "PyBEL Test Thorough",
    METADATA_DESCRIPTION: "Statements made up to contain many conceivable variants of nodes from BEL",
    METADATA_VERSION: "1.0.0",
    METADATA_COPYRIGHT: "Copyright (c) Charles Tapley Hoyt. All Rights Reserved.",
    METADATA_AUTHORS: "Charles Tapley Hoyt",
    METADATA_LICENSES: "WTF License",
    METADATA_CONTACT: "cthoyt@gmail.com",
}

citation_1 = citation_dict(namespace=CITATION_TYPE_PUBMED, identifier="123455")
citation_2 = citation_dict(namespace=CITATION_TYPE_PUBMED, identifier="123456")

evidence_1 = "Evidence 1"
dummy_evidence = "These are mostly made up"

akt1 = hgnc(name="AKT1")
egfr = hgnc(name="EGFR")
fadd = hgnc(name="FADD")
casp8 = hgnc(name="CASP8")
mia = hgnc(name="MIA")

il6 = Protein("HGNC", "IL6")

adgrb1 = Protein(namespace="HGNC", name="ADGRB1")
adgrb2 = Protein(namespace="HGNC", name="ADGRB2")
adgrb_complex = ComplexAbundance([adgrb1, adgrb2])
achlorhydria = Pathology(namespace="MESHD", name="Achlorhydria")

akt1_rna = akt1.get_rna()
akt1_gene = akt1_rna.get_gene()
akt_methylated = akt1_gene.with_variants(GeneModification("Me"))
akt1_phe_508_del = akt1_gene.with_variants(Hgvs("p.Phe508del"))

cftr = hgnc(name="CFTR")
cftr_protein_unspecified_variant = cftr.with_variants(HgvsUnspecified())
cftr_protein_phe_508_del = cftr.with_variants(Hgvs("p.Phe508del"))

adenocarcinoma = Pathology("MESHD", "Adenocarcinoma")
interleukin_23_complex = NamedComplexAbundance("GO", "interleukin-23 complex")

oxygen_atom = Abundance(namespace="CHEBI", name="oxygen atom")
hydrogen_peroxide = Abundance("CHEBI", "hydrogen peroxide")

tmprss2_gene = Gene("HGNC", "TMPRSS2")

tmprss2_erg_gene_fusion = GeneFusion(
    partner_5p=tmprss2_gene,
    range_5p=EnumeratedFusionRange("c", 1, 79),
    partner_3p=Gene("HGNC", "ERG"),
    range_3p=EnumeratedFusionRange("c", 312, 5034),
)

bcr_jak2_gene_fusion = GeneFusion(
    partner_5p=Gene("HGNC", "BCR"),
    range_5p=EnumeratedFusionRange("c", "?", 1875),
    partner_3p=Gene("HGNC", "JAK2"),
    range_3p=EnumeratedFusionRange("c", 2626, "?"),
)

chchd4_aifm1_gene_fusion = GeneFusion(
    partner_5p=Gene("HGNC", "CHCHD4"),
    partner_3p=Gene("HGNC", "AIFM1"),
)

tmprss2_erg_protein_fusion = ProteinFusion(
    partner_5p=Protein("HGNC", "TMPRSS2"),
    range_5p=EnumeratedFusionRange("p", 1, 79),
    partner_3p=Protein("HGNC", "ERG"),
    range_3p=EnumeratedFusionRange("p", 312, 5034),
)

bcr_jak2_protein_fusion = ProteinFusion(
    partner_5p=Protein("HGNC", "BCR"),
    range_5p=EnumeratedFusionRange("p", "?", 1875),
    partner_3p=Protein("HGNC", "JAK2"),
    range_3p=EnumeratedFusionRange("p", 2626, "?"),
)

chchd4_aifm1_protein_fusion = ProteinFusion(Protein("HGNC", "CHCHD4"), Protein("HGNC", "AIFM1"))

bcr_jak2_rna_fusion = RnaFusion(
    partner_5p=Rna("HGNC", "BCR"),
    range_5p=EnumeratedFusionRange("r", "?", 1875),
    partner_3p=Rna("HGNC", "JAK2"),
    range_3p=EnumeratedFusionRange("r", 2626, "?"),
)

chchd4_aifm1_rna_fusion = RnaFusion(partner_5p=Rna("HGNC", "CHCHD4"), partner_3p=Rna("HGNC", "AIFM1"))

tmprss2_erg_rna_fusion = RnaFusion(
    partner_5p=Rna("HGNC", "TMPRSS2"),
    range_5p=EnumeratedFusionRange("r", 1, 79),
    partner_3p=Rna("HGNC", "ERG"),
    range_3p=EnumeratedFusionRange("r", 312, 5034),
)
tmprss2_erg_rna_fusion_unspecified = RnaFusion(partner_5p=Rna("HGNC", "TMPRSS2"), partner_3p=Rna("HGNC", "ERG"))

BEL_THOROUGH_NODES = {
    oxygen_atom,
    tmprss2_erg_rna_fusion,
    tmprss2_erg_rna_fusion_unspecified,
    akt_methylated,
    bcr_jak2_rna_fusion,
    chchd4_aifm1_rna_fusion,
    akt1_gene,
    akt1_phe_508_del,
    akt1,
    Gene("HGNC", "AKT1", variants=Hgvs("c.308G>A")),
    tmprss2_erg_gene_fusion,
    Gene(
        "HGNC",
        "AKT1",
        variants=[Hgvs("c.1521_1523delCTT"), Hgvs("c.308G>A"), Hgvs("p.Phe508del")],
    ),
    MicroRna("HGNC", "MIR21"),
    bcr_jak2_gene_fusion,
    Gene("HGNC", "CFTR", variants=Hgvs("c.1521_1523delCTT")),
    Gene("HGNC", "CFTR"),
    Gene("HGNC", "CFTR", variants=Hgvs("g.117199646_117199648delCTT")),
    Gene("HGNC", "CFTR", variants=Hgvs("c.1521_1523delCTT")),
    Protein("HGNC", "AKT1", variants=ProteinModification("Ph", "Ser", 473)),
    MicroRna("HGNC", "MIR21", variants=Hgvs("p.Phe508del")),
    Protein("HGNC", "AKT1", variants=Hgvs("p.C40*")),
    Protein("HGNC", "AKT1", variants=[Hgvs("p.Ala127Tyr"), ProteinModification("Ph", "Ser")]),
    chchd4_aifm1_gene_fusion,
    tmprss2_erg_protein_fusion,
    Protein("HGNC", "AKT1", variants=Hgvs("p.Arg1851*")),
    bcr_jak2_protein_fusion,
    Protein("HGNC", "AKT1", variants=Hgvs("p.40*")),
    chchd4_aifm1_protein_fusion,
    Protein("HGNC", "CFTR", variants=HgvsReference()),
    cftr,
    egfr,
    cftr_protein_unspecified_variant,
    adenocarcinoma,
    cftr_protein_phe_508_del,
    Protein("HGNC", "MIA", variants=Fragment(5, 20)),
    mia,
    interleukin_23_complex,
    Protein("HGNC", "MIA", variants=Fragment(1, "?")),
    Protein("HGNC", "MIA", variants=Fragment()),
    Protein("HGNC", "MIA", variants=Fragment(description="55kD")),
    Protein("HGNC", "CFTR", variants=Hgvs("p.Gly576Ala")),
    akt1_rna,
    Rna("HGNC", "AKT1", variants=[Hgvs("c.1521_1523delCTT"), Hgvs("p.Phe508del")]),
    Gene("HGNC", "NCF1"),
    ComplexAbundance([Gene("HGNC", "NCF1"), Protein("HGNC", "HBP1")]),
    Protein("HGNC", "HBP1"),
    ComplexAbundance([Protein("HGNC", "FOS"), Protein("HGNC", "JUN")]),
    Protein("HGNC", "FOS"),
    Protein("HGNC", "JUN"),
    Rna("HGNC", "CFTR", variants=Hgvs("r.1521_1523delcuu")),
    Rna("HGNC", "CFTR"),
    Rna("HGNC", "CFTR", variants=Hgvs("r.1653_1655delcuu")),
    CompositeAbundance([interleukin_23_complex, il6]),
    il6,
    BiologicalProcess("GO", "cell cycle arrest"),
    hydrogen_peroxide,
    Protein("HGNC", "CAT"),
    Gene("HGNC", "CAT"),
    Protein("HGNC", "HMGCR"),
    BiologicalProcess("GO", "cholesterol biosynthetic process"),
    Gene("HGNC", "APP", variants=Hgvs("c.275341G>C")),
    Gene("HGNC", "APP"),
    Pathology("MESHD", "Alzheimer Disease"),
    ComplexAbundance([Protein("HGNC", "F3"), Protein("HGNC", "F7")]),
    Protein("HGNC", "F3"),
    Protein("HGNC", "F7"),
    Protein("HGNC", "F9"),
    Protein("HGNC", "GSK3B", variants=ProteinModification("Ph", "Ser", 9)),
    Protein("HGNC", "GSK3B"),
    Pathology("MESHD", "Psoriasis"),
    Pathology("MESHD", "Skin Diseases"),
    Reaction(
        reactants=[
            Abundance("CHEBI", "(3S)-3-hydroxy-3-methylglutaryl-CoA"),
            Abundance("CHEBI", "NADPH"),
            Abundance("CHEBI", "hydron"),
        ],
        products=[Abundance("CHEBI", "NADP(+)"), Abundance("CHEBI", "mevalonate")],
    ),
    Abundance("CHEBI", "(3S)-3-hydroxy-3-methylglutaryl-CoA"),
    Abundance("CHEBI", "NADPH"),
    Abundance("CHEBI", "hydron"),
    Abundance("CHEBI", "mevalonate"),
    Abundance("CHEBI", "NADP(+)"),
    Abundance("CHEBI", "nitric oxide"),
    ComplexAbundance([Protein("HGNC", "ITGAV"), Protein("HGNC", "ITGB3")]),
    Protein("HGNC", "ITGAV"),
    Protein("HGNC", "ITGB3"),
    Protein("HGNC", "FADD"),
    Abundance("TESTNS2", "Abeta_42"),
    Protein("TESTNS2", "GSK3 Family"),
    Protein("HGNC", "PRKCA"),
    Protein("HGNC", "CDK5"),
    Protein("HGNC", "CASP8"),
    Protein(
        "HGNC",
        "AKT1",
        variants=ProteinModification(namespace="TESTNS2", name="PhosRes", code="Ser", position=473),
    ),
    Protein("HGNC", "HRAS", variants=ProteinModification("Palm")),
    BiologicalProcess("GO", "apoptotic process"),
    CompositeAbundance(
        [
            Abundance("TESTNS2", "Abeta_42"),
            Protein("HGNC", "CASP8"),
            Protein("HGNC", "FADD"),
        ]
    ),
    Reaction(
        reactants=[Protein("HGNC", "CDK5R1")],
        products=[Protein("HGNC", "CDK5")],
    ),
    Protein("HGNC", "PRKCB"),
    NamedComplexAbundance("TESTNS2", "AP-1 Complex"),
    Protein("HGNC", "PRKCE"),
    Protein("HGNC", "PRKCD"),
    Protein("TESTNS2", "CAPN Family"),
    Gene("TESTNS2", "AKT1 ortholog"),
    Protein("HGNC", "HRAS"),
    Protein("HGNC", "CDK5R1"),
    Protein("TESTNS2", "PRKC"),
    BiologicalProcess("GO", "neuron apoptotic process"),
    Protein("HGNC", "MAPT", variants=ProteinModification("Ph")),
    Protein("HGNC", "MAPT"),
    Gene("HGNC", "ARRDC2"),
    Gene("HGNC", "ARRDC3"),
    Gene("dbSNP", "rs123456"),
}

BEL_THOROUGH_EDGES = [
    (
        Gene("HGNC", "AKT1", variants=Hgvs("p.Phe508del")),
        akt1,
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: DIRECTLY_DECREASES,
        },
    ),
    (
        akt1,
        Protein("HGNC", "AKT1", variants=ProteinModification("Ph", "Ser", 473)),
        {
            RELATION: HAS_VARIANT,
        },
    ),
    (
        akt1,
        Protein("HGNC", "AKT1", variants=Hgvs("p.C40*")),
        {
            RELATION: HAS_VARIANT,
        },
    ),
    (
        akt1,
        Protein(
            "HGNC",
            "AKT1",
            variants=[Hgvs("p.Ala127Tyr"), ProteinModification("Ph", "Ser")],
        ),
        {
            RELATION: HAS_VARIANT,
        },
    ),
    (
        akt1,
        Protein(
            "HGNC",
            "AKT1",
            variants=[Hgvs("p.Ala127Tyr"), ProteinModification("Ph", "Ser")],
        ),
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: DIRECTLY_DECREASES,
            SOURCE_MODIFIER: {LOCATION: {NAMESPACE: "GO", NAME: "intracellular"}},
            TARGET_MODIFIER: {LOCATION: {NAMESPACE: "GO", NAME: "intracellular"}},
        },
    ),
    (
        akt1,
        Protein("HGNC", "AKT1", variants=Hgvs("p.Arg1851*")),
        {
            RELATION: HAS_VARIANT,
        },
    ),
    (
        akt1,
        Protein("HGNC", "AKT1", variants=Hgvs("p.40*")),
        {
            RELATION: HAS_VARIANT,
        },
    ),
    (
        akt1,
        Protein("HGNC", "MIA", variants=Fragment()),
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: INCREASES,
            SOURCE_MODIFIER: {MODIFIER: DEGRADATION},
        },
    ),
    (
        akt1,
        Protein("HGNC", "CFTR", variants=Hgvs("p.Gly576Ala")),
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: INCREASES,
        },
    ),
    (
        akt1,
        Rna("HGNC", "CFTR", variants=Hgvs("r.1521_1523delcuu")),
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: INCREASES,
            SOURCE_MODIFIER: {MODIFIER: ACTIVITY, EFFECT: activity_mapping["kin"]},
        },
    ),
    (
        akt1,
        Rna("HGNC", "CFTR", variants=Hgvs("r.1653_1655delcuu")),
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: INCREASES,
            SOURCE_MODIFIER: {MODIFIER: ACTIVITY},
        },
    ),
    (
        akt1,
        egfr,
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: INCREASES,
            SOURCE_MODIFIER: {
                MODIFIER: ACTIVITY,
                EFFECT: activity_mapping["cat"],
            },
            TARGET_MODIFIER: {MODIFIER: DEGRADATION},
        },
    ),
    (
        akt1,
        egfr,
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: INCREASES,
            SOURCE_MODIFIER: {
                MODIFIER: ACTIVITY,
                EFFECT: activity_mapping["kin"],
            },
            TARGET_MODIFIER: secretion(),
        },
    ),
    (
        Gene("HGNC", "AKT1", variants=Hgvs("c.308G>A")),
        tmprss2_erg_gene_fusion,
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: CAUSES_NO_CHANGE,
        },
    ),
    (
        Gene("HGNC", "AKT1", variants=Hgvs("c.308G>A")),
        Gene(
            "HGNC",
            "AKT1",
            variants=[Hgvs("c.1521_1523delCTT"), Hgvs("c.308G>A"), Hgvs("p.Phe508del")],
        ),
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: INCREASES,
            SOURCE_MODIFIER: {LOCATION: {NAMESPACE: "GO", NAME: "intracellular"}},
        },
    ),
    (
        MicroRna("HGNC", "MIR21"),
        bcr_jak2_gene_fusion,
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: DIRECTLY_INCREASES,
        },
    ),
    (
        MicroRna("HGNC", "MIR21"),
        Protein("HGNC", "AKT1", variants=ProteinModification("Ph", "Ser", 473)),
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: DECREASES,
            SOURCE_MODIFIER: {LOCATION: {NAMESPACE: "GO", NAME: "intracellular"}},
        },
    ),
    (
        MicroRna("HGNC", "MIR21"),
        MicroRna("HGNC", "MIR21", variants=Hgvs("p.Phe508del")),
        {
            RELATION: HAS_VARIANT,
        },
    ),
    (
        Gene("HGNC", "CFTR", variants=Hgvs("c.1521_1523delCTT")),
        akt1,
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: INCREASES,
            TARGET_MODIFIER: {MODIFIER: DEGRADATION},
        },
    ),
    (
        Gene("HGNC", "CFTR"),
        Gene("HGNC", "CFTR", variants=Hgvs("c.1521_1523delCTT")),
        {
            RELATION: HAS_VARIANT,
        },
    ),
    (
        Gene("HGNC", "CFTR"),
        Gene("HGNC", "CFTR", variants=Hgvs("g.117199646_117199648delCTT")),
        {
            RELATION: HAS_VARIANT,
        },
    ),
    (
        Gene("HGNC", "CFTR"),
        Gene("HGNC", "CFTR", variants=Hgvs("c.1521_1523delCTT")),
        {
            RELATION: HAS_VARIANT,
        },
    ),
    (
        Gene("HGNC", "CFTR", variants=Hgvs("g.117199646_117199648delCTT")),
        Gene("HGNC", "CFTR", variants=Hgvs("c.1521_1523delCTT")),
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: INCREASES,
        },
    ),
    (
        MicroRna("HGNC", "MIR21", variants=Hgvs("p.Phe508del")),
        Protein("HGNC", "AKT1", variants=Hgvs("p.C40*")),
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: INCREASES,
            SOURCE_MODIFIER: {LOCATION: {NAMESPACE: "GO", NAME: "intracellular"}},
        },
    ),
    (
        chchd4_aifm1_gene_fusion,
        tmprss2_erg_protein_fusion,
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: INCREASES,
        },
    ),
    (
        Protein("HGNC", "AKT1", variants=Hgvs("p.Arg1851*")),
        bcr_jak2_protein_fusion,
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: INCREASES,
        },
    ),
    (
        Protein("HGNC", "AKT1", variants=Hgvs("p.40*")),
        chchd4_aifm1_protein_fusion,
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: INCREASES,
        },
    ),
    (
        Protein("HGNC", "CFTR", variants=HgvsReference()),
        egfr,
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: INCREASES,
            TARGET_MODIFIER: translocation(INTRACELLULAR, CELL_SURFACE),
        },
    ),
    (
        cftr,
        Protein("HGNC", "CFTR", variants=Hgvs("=")),
        {
            RELATION: HAS_VARIANT,
        },
    ),
    (
        cftr,
        Protein("HGNC", "CFTR", variants=Hgvs("?")),
        {
            RELATION: HAS_VARIANT,
        },
    ),
    (
        cftr,
        Protein("HGNC", "CFTR", variants=Hgvs("p.Phe508del")),
        {
            RELATION: HAS_VARIANT,
        },
    ),
    (
        cftr,
        Protein("HGNC", "CFTR", variants=Hgvs("p.Gly576Ala")),
        {
            RELATION: HAS_VARIANT,
        },
    ),
    (
        mia,
        Protein("HGNC", "MIA", variants=Fragment(5, 20)),
        {
            RELATION: HAS_VARIANT,
        },
    ),
    (
        mia,
        Protein("HGNC", "MIA", variants=Fragment(1, "?")),
        {
            RELATION: HAS_VARIANT,
        },
    ),
    (
        mia,
        Protein("HGNC", "MIA", variants=Fragment()),
        {
            RELATION: HAS_VARIANT,
        },
    ),
    (
        mia,
        Protein("HGNC", "MIA", variants=Fragment(description="55kD")),
        {
            RELATION: HAS_VARIANT,
        },
    ),
    (
        akt1_rna,
        Rna("HGNC", "AKT1", variants=[Hgvs("c.1521_1523delCTT"), Hgvs("p.Phe508del")]),
        {
            RELATION: HAS_VARIANT,
        },
    ),
    (
        akt1_rna,
        akt1,
        {
            RELATION: TRANSLATED_TO,
        },
    ),
    (
        Gene("HGNC", "APP"),
        Gene("HGNC", "APP", variants=Hgvs("c.275341G>C")),
        {
            RELATION: HAS_VARIANT,
        },
    ),
    (
        Protein("HGNC", "F3"),
        ComplexAbundance([Protein("HGNC", "F3"), Protein("HGNC", "F7")]),
        {
            RELATION: PART_OF,
        },
    ),
    (
        Protein("HGNC", "F7"),
        ComplexAbundance([Protein("HGNC", "F3"), Protein("HGNC", "F7")]),
        {
            RELATION: PART_OF,
        },
    ),
    (
        Protein("HGNC", "GSK3B"),
        Protein("HGNC", "GSK3B", variants=ProteinModification("Ph", "Ser", 9)),
        {
            RELATION: HAS_VARIANT,
        },
    ),
    (
        Pathology("MESHD", "Psoriasis"),
        Pathology("MESHD", "Skin Diseases"),
        {
            RELATION: IS_A,
        },
    ),
    (
        Protein("HGNC", "HBP1"),
        ComplexAbundance([Gene("HGNC", "NCF1"), Protein("HGNC", "HBP1")]),
        {
            RELATION: PART_OF,
        },
    ),
    (
        Gene("HGNC", "NCF1"),
        ComplexAbundance([Gene("HGNC", "NCF1"), Protein("HGNC", "HBP1")]),
        {
            RELATION: PART_OF,
        },
    ),
    (
        Protein("HGNC", "FOS"),
        ComplexAbundance([Protein("HGNC", "FOS"), Protein("HGNC", "JUN")]),
        {
            RELATION: PART_OF,
        },
    ),
    (
        Protein("HGNC", "JUN"),
        ComplexAbundance([Protein("HGNC", "FOS"), Protein("HGNC", "JUN")]),
        {
            RELATION: PART_OF,
        },
    ),
    (
        Rna("HGNC", "CFTR"),
        Rna("HGNC", "CFTR", variants=Hgvs("r.1521_1523delcuu")),
        {
            RELATION: HAS_VARIANT,
        },
    ),
    (
        Rna("HGNC", "CFTR"),
        Rna("HGNC", "CFTR", variants=Hgvs("r.1653_1655delcuu")),
        {
            RELATION: HAS_VARIANT,
        },
    ),
    (
        il6,
        CompositeAbundance([interleukin_23_complex, il6]),
        {
            RELATION: PART_OF,
        },
    ),
    (
        interleukin_23_complex,
        CompositeAbundance([interleukin_23_complex, il6]),
        {
            RELATION: PART_OF,
        },
    ),
    (
        Protein("HGNC", "CFTR", variants=Hgvs("?")),
        Pathology("MESHD", "Adenocarcinoma"),
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: INCREASES,
        },
    ),
    (
        Rna("HGNC", "AKT1", variants=[Hgvs("c.1521_1523delCTT"), Hgvs("p.Phe508del")]),
        tmprss2_erg_rna_fusion,
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: DIRECTLY_INCREASES,
        },
    ),
    (
        RnaFusion(Rna("HGNC", "TMPRSS2"), Rna("HGNC", "ERG")),
        ComplexAbundance([Gene("HGNC", "NCF1"), Protein("HGNC", "HBP1")]),
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: INCREASES,
        },
    ),
    (
        Protein("HGNC", "MIA", variants=Fragment(5, 20)),
        NamedComplexAbundance("GO", "interleukin-23 complex"),
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: INCREASES,
            TARGET_MODIFIER: secretion(),
        },
    ),
    (
        Protein("HGNC", "MIA", variants=Fragment(1, "?")),
        egfr,
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: INCREASES,
            TARGET_MODIFIER: {
                MODIFIER: TRANSLOCATION,
                EFFECT: {
                    FROM_LOC: {NAMESPACE: "GO", NAME: "cell surface"},
                    TO_LOC: {NAMESPACE: "GO", NAME: "endosome"},
                },
            },
        },
    ),
    (
        akt1_rna,
        egfr,
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: INCREASES,
            TARGET_MODIFIER: {
                MODIFIER: TRANSLOCATION,
                EFFECT: {
                    FROM_LOC: {NAMESPACE: "GO", NAME: "cell surface"},
                    TO_LOC: {NAMESPACE: "GO", NAME: "endosome"},
                },
            },
        },
    ),
    (
        RnaFusion(
            Rna("HGNC", "CHCHD4"),
            Rna("HGNC", "AIFM1"),
        ),
        ComplexAbundance([Protein("HGNC", "FOS"), Protein("HGNC", "JUN")]),
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: INCREASES,
        },
    ),
    (
        CompositeAbundance([interleukin_23_complex, il6]),
        BiologicalProcess("GO", "cell cycle arrest"),
        {
            EVIDENCE: dummy_evidence,
            CITATION: citation_1,
            RELATION: DECREASES,
        },
    ),
    (
        Protein("HGNC", "CAT"),
        hydrogen_peroxide,
        {
            EVIDENCE: "These were all explicitly stated in the BEL 2.0 Specification",
            CITATION: citation_2,
            RELATION: DIRECTLY_DECREASES,
            SOURCE_MODIFIER: {LOCATION: {NAMESPACE: "GO", NAME: "intracellular"}},
        },
    ),
    (
        Gene("HGNC", "CAT"),
        hydrogen_peroxide,
        {
            EVIDENCE: "These were all explicitly stated in the BEL 2.0 Specification",
            CITATION: citation_2,
            RELATION: DIRECTLY_DECREASES,
            SOURCE_MODIFIER: {LOCATION: {NAMESPACE: "GO", NAME: "intracellular"}},
        },
    ),
    (
        Protein("HGNC", "HMGCR"),
        BiologicalProcess("GO", "cholesterol biosynthetic process"),
        {
            EVIDENCE: "These were all explicitly stated in the BEL 2.0 Specification",
            CITATION: citation_2,
            RELATION: RATE_LIMITING_STEP_OF,
            SOURCE_MODIFIER: {MODIFIER: ACTIVITY, EFFECT: activity_mapping["cat"]},
        },
    ),
    (
        Gene("HGNC", "APP", variants=Hgvs("c.275341G>C")),
        Pathology("MESHD", "Alzheimer Disease"),
        {
            EVIDENCE: "These were all explicitly stated in the BEL 2.0 Specification",
            CITATION: citation_2,
            RELATION: CAUSES_NO_CHANGE,
        },
    ),
    (
        ComplexAbundance([Protein("HGNC", "F3"), Protein("HGNC", "F7")]),
        Protein("HGNC", "F9"),
        {
            EVIDENCE: "These were all explicitly stated in the BEL 2.0 Specification",
            CITATION: citation_2,
            RELATION: REGULATES,
            SOURCE_MODIFIER: {MODIFIER: ACTIVITY, EFFECT: activity_mapping["pep"]},
            TARGET_MODIFIER: {MODIFIER: ACTIVITY, EFFECT: activity_mapping["pep"]},
        },
    ),
    (
        Protein("HGNC", "GSK3B", variants=ProteinModification("Ph", "Ser", 9)),
        Protein("HGNC", "GSK3B"),
        {
            EVIDENCE: "These were all explicitly stated in the BEL 2.0 Specification",
            CITATION: citation_2,
            RELATION: POSITIVE_CORRELATION,
            TARGET_MODIFIER: {MODIFIER: ACTIVITY, EFFECT: activity_mapping["kin"]},
        },
    ),
    (
        Protein("HGNC", "GSK3B"),
        Protein("HGNC", "GSK3B", variants=ProteinModification("Ph", "Ser", 9)),
        {
            EVIDENCE: "These were all explicitly stated in the BEL 2.0 Specification",
            CITATION: citation_2,
            RELATION: POSITIVE_CORRELATION,
            SOURCE_MODIFIER: {MODIFIER: ACTIVITY, EFFECT: activity_mapping["kin"]},
        },
    ),
    (
        Reaction(
            reactants=(
                Abundance("CHEBI", "(3S)-3-hydroxy-3-methylglutaryl-CoA"),
                Abundance("CHEBI", "NADPH"),
                Abundance("CHEBI", "hydron"),
            ),
            products=(Abundance("CHEBI", "NADP(+)"), Abundance("CHEBI", "mevalonate")),
        ),
        Abundance("CHEBI", "(3S)-3-hydroxy-3-methylglutaryl-CoA"),
        {
            RELATION: HAS_REACTANT,
        },
    ),
    (
        Reaction(
            reactants=(
                Abundance("CHEBI", "(3S)-3-hydroxy-3-methylglutaryl-CoA"),
                Abundance("CHEBI", "NADPH"),
                Abundance("CHEBI", "hydron"),
            ),
            products=(Abundance("CHEBI", "NADP(+)"), Abundance("CHEBI", "mevalonate")),
        ),
        Abundance("CHEBI", "NADPH"),
        {
            RELATION: HAS_REACTANT,
        },
    ),
    (
        Reaction(
            reactants=(
                Abundance("CHEBI", "(3S)-3-hydroxy-3-methylglutaryl-CoA"),
                Abundance("CHEBI", "NADPH"),
                Abundance("CHEBI", "hydron"),
            ),
            products=(Abundance("CHEBI", "NADP(+)"), Abundance("CHEBI", "mevalonate")),
        ),
        Abundance("CHEBI", "hydron"),
        {
            RELATION: HAS_REACTANT,
        },
    ),
    (
        Reaction(
            reactants=(
                Abundance("CHEBI", "(3S)-3-hydroxy-3-methylglutaryl-CoA"),
                Abundance("CHEBI", "NADPH"),
                Abundance("CHEBI", "hydron"),
            ),
            products=(Abundance("CHEBI", "NADP(+)"), Abundance("CHEBI", "mevalonate")),
        ),
        Abundance("CHEBI", "mevalonate"),
        {
            RELATION: HAS_PRODUCT,
        },
    ),
    (
        Reaction(
            reactants=(
                Abundance("CHEBI", "(3S)-3-hydroxy-3-methylglutaryl-CoA"),
                Abundance("CHEBI", "NADPH"),
                Abundance("CHEBI", "hydron"),
            ),
            products=(Abundance("CHEBI", "NADP(+)"), Abundance("CHEBI", "mevalonate")),
        ),
        Abundance("CHEBI", "NADP(+)"),
        {
            RELATION: HAS_PRODUCT,
        },
    ),
    (
        Reaction(
            reactants=(
                Abundance("CHEBI", "(3S)-3-hydroxy-3-methylglutaryl-CoA"),
                Abundance("CHEBI", "NADPH"),
                Abundance("CHEBI", "hydron"),
            ),
            products=(Abundance("CHEBI", "NADP(+)"), Abundance("CHEBI", "mevalonate")),
        ),
        BiologicalProcess("GO", "cholesterol biosynthetic process"),
        {
            EVIDENCE: "These were all explicitly stated in the BEL 2.0 Specification",
            CITATION: citation_2,
            RELATION: SUBPROCESS_OF,
        },
    ),
    (
        Abundance("CHEBI", "nitric oxide"),
        ComplexAbundance([Protein("HGNC", "ITGAV"), Protein("HGNC", "ITGB3")]),
        {
            EVIDENCE: "These were all explicitly stated in the BEL 2.0 Specification",
            CITATION: citation_2,
            RELATION: INCREASES,
            TARGET_MODIFIER: {
                MODIFIER: TRANSLOCATION,
                EFFECT: {
                    FROM_LOC: compartment_mapping["intracellular"],
                    TO_LOC: compartment_mapping["cell surface"],
                },
            },
        },
    ),
    (
        Protein("HGNC", "ITGAV"),
        ComplexAbundance([Protein("HGNC", "ITGAV"), Protein("HGNC", "ITGB3")]),
        {
            RELATION: PART_OF,
        },
    ),
    (
        Protein("HGNC", "ITGB3"),
        ComplexAbundance([Protein("HGNC", "ITGAV"), Protein("HGNC", "ITGB3")]),
        {
            RELATION: PART_OF,
        },
    ),
    (
        Gene("HGNC", "ARRDC2"),
        Gene("HGNC", "ARRDC3"),
        {
            RELATION: EQUIVALENT_TO,
        },
    ),
    (
        Gene("HGNC", "ARRDC3"),
        Gene("HGNC", "ARRDC2"),
        {
            RELATION: EQUIVALENT_TO,
            CITATION: citation_2,
            EVIDENCE: "These were all explicitly stated in the BEL 2.0 Specification",
        },
    ),
    (
        Gene("dbSNP", "rs123456"),
        Gene("HGNC", "CFTR", variants=Hgvs("c.1521_1523delCTT")),
        {
            RELATION: ASSOCIATION,
            CITATION: citation_2,
            EVIDENCE: "These were all explicitly stated in the BEL 2.0 Specification",
        },
    ),
    (
        Gene("HGNC", "CFTR", variants=Hgvs("c.1521_1523delCTT")),
        Gene("dbSNP", "rs123456"),
        {
            RELATION: ASSOCIATION,
            CITATION: citation_2,
            EVIDENCE: "These were all explicitly stated in the BEL 2.0 Specification",
        },
    ),
]



# -*- coding: utf-8 -*-

"""Tests for PyBEL utilities."""

import unittest

from pybel.exceptions import PlaceholderAminoAcidWarning
from pybel.parser.modifiers.constants import amino_acid
from pybel.parser.utils import nest
from pybel.utils import expand_dict, flatten_dict, tokenize_version


class TestTokenizeVersion(unittest.TestCase):
    """Test tokenization of version strings."""

    def test_simple(self):
        """Test the simplest version string case."""
        version_str = "0.1.2"
        version_tuple = 0, 1, 2
        self.assertEqual(version_tuple, tokenize_version(version_str))

    def test_long(self):
        """Test when the version pieces have more than 1 digit."""
        version_str = "0.12.20"
        version_tuple = 0, 12, 20
        self.assertEqual(version_tuple, tokenize_version(version_str))

    def test_dev(self):
        """Test when there's a dash after."""
        version_str = "0.1.2-dev"
        version_tuple = 0, 1, 2
        self.assertEqual(version_tuple, tokenize_version(version_str))


class TestRandom(unittest.TestCase):
    def test_nest_failure(self):
        with self.assertRaises(ValueError):
            nest()

    def test_bad_aminoAcid(self):
        with self.assertRaises(PlaceholderAminoAcidWarning):
            amino_acid.parseString("X")


class TestUtils(unittest.TestCase):
    def test_expand_dict(self):
        flat_dict = {
            "k1": "v1",
            "k2_k2a": "v2",
            "k2_k2b": "v3",
            "k2_k2c_k2ci": "v4",
            "k2_k2c_k2cii": "v5",
        }

        expected_dict = {
            "k1": "v1",
            "k2": {"k2a": "v2", "k2b": "v3", "k2c": {"k2ci": "v4", "k2cii": "v5"}},
        }

        self.assertEqual(expected_dict, expand_dict(flat_dict))

    def test_flatten_dict(self):
        d = {"A": 5, "B": "b", "C": {"D": "d", "E": "e"}}

        expected = {"A": 5, "B": "b", "C_D": "d", "C_E": "e"}
        self.assertEqual(expected, flatten_dict(d))

    def test_flatten_dict_withLists(self):
        d = {"A": 5, "B": "b", "C": {"D": ["d", "delta"], "E": "e"}}

        expected = {"A": 5, "B": "b", "C_D": "d,delta", "C_E": "e"}
        self.assertEqual(expected, flatten_dict(d))



# -*- coding: utf-8 -*-

"""Summary functions for BEL graphs."""

from . import edge_summary, errors, node_summary, provenance
from .edge_summary import *
from .errors import *
from .node_summary import *
from .provenance import *

__all__ = [k for k in locals() if not k.startswith("_")]



# -*- coding: utf-8 -*-

"""The :mod:`pybel.struct` module houses functions for handling the main data structure in PyBEL.

Because BEL expresses how biological entities interact within many
different contexts, with descriptive annotations, PyBEL represents data as a directed multi-graph by sub-classing the
:class:`networkx.MultiDiGraph`. Each node is an instance of a subclass of the :class:`pybel.dsl.BaseEntity` and each
edge has a stable key and associated data dictionary for storing relevant contextual information.

The graph contains metadata for the PyBEL version, the BEL script metadata, the namespace definitions, the
annotation definitions, and the warnings produced in analysis. Like any :mod:`networkx` graph, all attributes of
a given object can be accessed through the :code:`graph` property, like in: :code:`my_graph.graph['my key']`.
Convenient property definitions are given for these attributes that are outlined in the documentation for
:class:`pybel.BELGraph`.

This allows for much easier programmatic access to answer more complicated questions, which can be written with python
code. Because the data structure is the same in Neo4J, the data can be directly exported with :func:`pybel.to_neo4j`.
Neo4J supports the Cypher querying language so that the same queries can be written in an elegant and simple way.
"""

from . import filters, graph, grouping, mutation, node_utils, operations, summary
from .filters import *
from .graph import *
from .grouping import *
from .mutation import *
from .node_utils import *
from .operations import *
from .pipeline import Pipeline
from .query import Query
from .summary import *



# -*- coding: utf-8 -*-

"""Tests for functions for building node predicates."""

import unittest

from pybel import BELGraph
from pybel.constants import GENE, PROTEIN
from pybel.dsl import bioprocess, gene, protein
from pybel.struct import filter_nodes
from pybel.struct.filters import invert_node_predicate
from pybel.struct.filters.node_predicate_builders import (
    build_node_graph_data_search,
    build_node_name_search,
    data_missing_key_builder,
    function_inclusion_filter_builder,
)
from pybel.testing.utils import n


class TestFunctionInclusionFilterBuilder(unittest.TestCase):
    """Tests for the function_inclusion_filter_builder function."""

    def test_type_error(self):
        """Test that a type error is thrown for an invalid argument type."""
        with self.assertRaises(TypeError):
            function_inclusion_filter_builder(5)

    def test_empty_list_error(self):
        """Test that a value error is thrown for an empty list."""
        with self.assertRaises(ValueError):
            function_inclusion_filter_builder([])

    def test_single(self):
        """Test building a node predicate with a single function."""
        f = function_inclusion_filter_builder(GENE)

        p1 = protein(n(), n())
        g1 = gene(n(), n())

        g = BELGraph()
        g.add_node_from_data(p1)
        g.add_node_from_data(g1)

        self.assertIn(p1, g)
        self.assertIn(g1, g)

        self.assertFalse(f(g, p1))
        self.assertTrue(f(g, g1))

        f = invert_node_predicate(f)

        self.assertTrue(f(g, p1))
        self.assertFalse(f(g, g1))

    def test_multiple(self):
        """Test building a node predicate with multiple functions."""
        f = function_inclusion_filter_builder([GENE, PROTEIN])

        p1 = protein(n(), n())
        g1 = gene(n(), n())
        b1 = bioprocess(n(), n())

        g = BELGraph()
        g.add_node_from_data(p1)
        g.add_node_from_data(g1)
        g.add_node_from_data(b1)

        self.assertIn(p1, g)
        self.assertIn(g1, g)
        self.assertIn(b1, g)

        self.assertTrue(f(g, p1))
        self.assertTrue(f(g, g1))
        self.assertFalse(f(g, b1))

        f = invert_node_predicate(f)

        self.assertFalse(f(g, p1))
        self.assertFalse(f(g, g1))
        self.assertTrue(f(g, b1))


class TestNodePredicateBuilders(unittest.TestCase):
    """Tests for node predicate builders."""

    def test_data_missing_key_builder(self):
        """Test the data_missing_key_builder function."""
        graph = BELGraph()
        p1 = protein("HGNC", n())
        p2 = protein("HGNC", n())
        graph.add_node_from_data(p1)
        graph.add_node_from_data(p2)

        key, other_key = "k1", "k2"

        data_missing_key = data_missing_key_builder(key)

        graph.nodes[p1][key] = n()
        graph.nodes[p2][other_key] = n()

        nodes = set(filter_nodes(graph, data_missing_key))

        self.assertNotIn(p1, nodes)
        self.assertIn(p2, nodes)

    def test_build_node_data_search(self):
        """Test build_node_data_search."""

        def test_key_predicate(datum):
            """Check the data is greater than zero.

            :rtype: bool
            """
            return 0 < datum

        key = n()

        data_predicate = build_node_graph_data_search(key, test_key_predicate)

        graph = BELGraph()

        p1 = protein("HGNC", n())
        graph.add_node_from_data(p1)
        graph.nodes[p1][key] = 0
        self.assertFalse(data_predicate(graph, p1))

        p2 = protein("HGNC", n())
        graph.add_node_from_data(p2)
        graph.nodes[p2][key] = 5
        self.assertTrue(data_predicate(graph, p2))

        p3 = protein("HGNC", n())
        graph.add_node_from_data(p3)
        self.assertFalse(data_predicate(graph, p3))

    def test_build_node_name_search(self):
        graph = BELGraph()
        p1 = protein("HGNC", "APP")
        graph.add_node_from_data(p1)
        p2 = protein("HGNC", "MAPK")
        graph.add_node_from_data(p2)
        node_name_search = build_node_name_search(query="APP")
        self.assertTrue(node_name_search(graph, p1))
        self.assertFalse(node_name_search(graph, p2))



# -*- coding: utf-8 -*-

"""An example describing EGF's effect on cellular processes.

.. code-block:: none

    SET Citation = {"PubMed","Clin Cancer Res 2003 Jul 9(7) 2416-25","12855613"}
    SET Evidence = "This induction was not seen either when LNCaP cells were treated with flutamide or conditioned medium were pretreated with antibody to the epidermal growth factor (EGF)"
    SET Species = 9606

    tscript(p(HGNC:AR)) increases p(HGNC:EGF)

    UNSET ALL

    SET Citation = {"PubMed","Int J Cancer 1998 Jul 3 77(1) 138-45","9639405"}
    SET Evidence = "DU-145 cells treated with 5000 U/ml of IFNgamma and IFN alpha, both reduced EGF production with IFN gamma reduction more significant."
    SET Species = 9606

    p(HGNC:IFNA1) decreases p(HGNC:EGF)
    p(HGNC:IFNG) decreases p(HGNC:EGF)

    UNSET ALL


    SET Citation = {"PubMed","DNA Cell Biol 2000 May 19(5) 253-63","10855792"}
    SET Evidence = "Although found predominantly in the cytoplasm and, less abundantly, in the nucleus, VCP can be translocated from the nucleus after stimulation with epidermal growth factor."
    SET Species = 9606

    p(HGNC:EGF) increases tloc(p(HGNC:VCP), GO:nucleus, GO:cytoplasm)

    UNSET ALL

    SET Citation = {"PubMed","J Clin Oncol 2003 Feb 1 21(3) 447-52","12560433"}
    SET Evidence = "Valosin-containing protein (VCP; also known as p97) has been shown to be associated with antiapoptotic function and metastasis via activation of the nuclear factor-kappaB signaling pathway."
    SET Species = 9606

    cat(p(HGNC:VCP)) increases tscript(complex(p(HGNC:NFKB1), p(HGNC:NFKB2), p(HGNC:REL), p(HGNC:RELA), p(HGNC:RELB)))
    tscript(complex(p(HGNC:NFKB1), p(HGNC:NFKB2), p(HGNC:REL), p(HGNC:RELA), p(HGNC:RELB))) decreases bp(MESHPP:Apoptosis)

    UNSET ALL
"""

from ..dsl import BiologicalProcess, ComplexAbundance, Protein, activity, translocation
from ..language import cytoplasm, nucleus
from ..resources import CHEBI_URL, CONFIDENCE_URL, GO_URL, HGNC_URL, SPECIES_PATTERN
from ..struct.graph import BELGraph

__all__ = [
    "egf_graph",
]

egf_graph = BELGraph(
    name="EGF Pathway",
    version="1.0.0",
    description="The downstream effects of EGF",
    authors="Charles Tapley Hoyt",
    contact="cthoyt@gmail.com",
)

egf_graph.namespace_url.update(
    {
        "hgnc": HGNC_URL,
        "chebi": CHEBI_URL,
        "go": GO_URL,
    }
)

egf_graph.annotation_url.update(
    {
        "Confidence": CONFIDENCE_URL,
    }
)

egf_graph.annotation_pattern.update(
    {
        "Species": SPECIES_PATTERN,
    }
)

ar = Protein(name="AR", namespace="hgnc")
egf = Protein(name="EGF", namespace="hgnc")
ifna1 = Protein(name="IFNA1", namespace="hgnc")
ifng = Protein(name="IFNG", namespace="hgnc")
vcp = Protein(name="VCP", namespace="hgnc")

nfkb1 = Protein(name="NFKB1", namespace="hgnc")
nfkb2 = Protein(name="NFKB2", namespace="hgnc")
rel = Protein(name="REL", namespace="hgnc")
rela = Protein(name="RELA", namespace="hgnc")
relb = Protein(name="RELB", namespace="hgnc")

nfkb_complex = ComplexAbundance([nfkb1, nfkb2, rel, rela, relb])

apoptosis = BiologicalProcess(namespace="go", name="apoptotic process", identifier="0006915")

egf_graph.add_increases(
    ar,
    egf,
    citation="12855613",
    evidence="This induction was not seen either when LNCaP cells were treated with flutamide or conditioned medium "
    "were pretreated with antibody to the epidermal growth factor (EGF)",
    annotations={"Species": "9606"},
    source_modifier=activity("tscript"),
)

egf_graph.add_decreases(
    ifna1,
    egf,
    citation="9639405",
    evidence="DU-145 cells treated with 5000 U/ml of IFNgamma and IFN alpha, both reduced EGF production with IFN "
    "gamma reduction more significant.",
    annotations={"Species": "9606"},
)

egf_graph.add_decreases(
    ifng,
    egf,
    citation="9639405",
    evidence="DU-145 cells treated with 5000 U/ml of IFNgamma and IFN alpha, both reduced EGF production with IFN "
    "gamma reduction more significant.",
    annotations={"Species": "9606"},
)

egf_graph.add_increases(
    egf,
    vcp,
    citation="10855792",
    evidence="Although found predominantly in the cytoplasm and, less abundantly, in the nucleus, VCP can be "
    "translocated from the nucleus after stimulation with epidermal growth factor.",
    annotations={"Species": "9606"},
    target_modifier=translocation(
        from_loc=nucleus,
        to_loc=cytoplasm,
    ),
)

egf_graph.add_increases(
    vcp,
    nfkb_complex,
    citation="12560433",
    evidence="Valosin-containing protein (VCP; also known as p97) has been shown to be associated with antiapoptotic"
    " function and metastasis via activation of the nuclear factor-kappaB signaling pathway.",
    annotations={"Species": "9606"},
    source_modifier=activity("cat"),
    target_modifier=activity("tscript"),
)

egf_graph.add_decreases(
    nfkb_complex,
    apoptosis,
    citation="12560433",
    evidence="Valosin-containing protein (VCP; also known as p97) has been shown to be associated with antiapoptotic "
    "function and metastasis via activation of the nuclear factor-kappaB signaling pathway.",
    annotations={"Species": "9606"},
    source_modifier=activity("tscript"),
)



# -*- coding: utf-8 -*-

"""Convenient dictionaries for mapping constants to DSL classes."""

from .node_classes import (
    Abundance,
    BiologicalProcess,
    ComplexAbundance,
    CompositeAbundance,
    Gene,
    GeneFusion,
    MicroRna,
    NamedComplexAbundance,
    Pathology,
    Population,
    Protein,
    ProteinFusion,
    Rna,
    RnaFusion,
)
from ..constants import (
    ABUNDANCE,
    BIOPROCESS,
    COMPLEX,
    COMPOSITE,
    GENE,
    MIRNA,
    PATHOLOGY,
    POPULATION,
    PROTEIN,
    RNA,
)

__all__ = [
    "FUNC_TO_DSL",
    "FUNC_TO_FUSION_DSL",
    "FUNC_TO_LIST_DSL",
]

FUNC_TO_DSL = {
    PROTEIN: Protein,
    RNA: Rna,
    MIRNA: MicroRna,
    GENE: Gene,
    PATHOLOGY: Pathology,
    BIOPROCESS: BiologicalProcess,
    COMPLEX: NamedComplexAbundance,
    ABUNDANCE: Abundance,
    POPULATION: Population,
}

FUNC_TO_FUSION_DSL = {
    GENE: GeneFusion,
    RNA: RnaFusion,
    PROTEIN: ProteinFusion,
}

FUNC_TO_LIST_DSL = {
    COMPLEX: ComplexAbundance,
    COMPOSITE: CompositeAbundance,
}



# -*- coding: utf-8 -*-

"""Tests for edge predicate builders."""

import unittest

from pybel.constants import CITATION, CITATION_AUTHORS, CITATION_TYPE_PUBMED
from pybel.language import citation_dict
from pybel.struct.filters.edge_predicate_builders import (
    build_author_inclusion_filter,
    build_pmid_inclusion_filter,
)

pmid1 = "1"
pmid2 = "2"
pmid3 = "3"

author1 = "1"
author2 = "2"
author3 = "3"


class TestEdgePredicateBuilders(unittest.TestCase):
    """Tests for edge predicate builders."""

    def test_build_pmid_inclusion_filter(self):
        """Test building a predicate for a single PubMed identifier."""
        pmid_inclusion_filter = build_pmid_inclusion_filter(pmid1)

        self.assertTrue(
            pmid_inclusion_filter(
                {
                    CITATION: citation_dict(namespace=CITATION_TYPE_PUBMED, identifier=pmid1),
                }
            )
        )

        self.assertFalse(
            pmid_inclusion_filter(
                {
                    CITATION: citation_dict(namespace=CITATION_TYPE_PUBMED, identifier=pmid2),
                }
            )
        )

    def test_build_pmid_set_inclusion_filter(self):
        """Test building a predicate for multiple PubMed identifiers."""
        pmids = {pmid1, pmid2}
        pmid_inclusion_filter = build_pmid_inclusion_filter(pmids)

        self.assertTrue(
            pmid_inclusion_filter(
                {
                    CITATION: citation_dict(namespace=CITATION_TYPE_PUBMED, identifier=pmid1),
                }
            )
        )

        self.assertTrue(
            pmid_inclusion_filter(
                {
                    CITATION: citation_dict(namespace=CITATION_TYPE_PUBMED, identifier=pmid2),
                }
            )
        )

        self.assertFalse(
            pmid_inclusion_filter(
                {
                    CITATION: citation_dict(namespace=CITATION_TYPE_PUBMED, identifier=pmid3),
                }
            )
        )

    def test_build_author_inclusion_filter(self):
        """Test building a predicate for a single author."""
        author_inclusion_filter = build_author_inclusion_filter(author1)

        self.assertTrue(
            author_inclusion_filter(
                {
                    CITATION: citation_dict(
                        namespace=CITATION_TYPE_PUBMED,
                        identifier=pmid3,
                        authors=[author1],
                    ),
                }
            )
        )

        self.assertTrue(
            author_inclusion_filter(
                {
                    CITATION: citation_dict(
                        namespace=CITATION_TYPE_PUBMED,
                        identifier=pmid3,
                        authors=[author1, author2],
                    ),
                }
            )
        )

        self.assertFalse(
            author_inclusion_filter(
                {
                    CITATION: citation_dict(
                        namespace=CITATION_TYPE_PUBMED,
                        identifier=pmid3,
                        authors=[author3],
                    ),
                }
            )
        )

    def test_build_author_set_inclusion_filter(self):
        """Test building a predicate for multiple authors."""
        author = {author1, author2}
        author_inclusion_filter = build_author_inclusion_filter(author)

        self.assertTrue(author_inclusion_filter({CITATION: {CITATION_AUTHORS: [author1]}}))

        self.assertTrue(author_inclusion_filter({CITATION: {CITATION_AUTHORS: [author1, author2]}}))

        self.assertFalse(author_inclusion_filter({CITATION: {CITATION_AUTHORS: [author3]}}))



# -*- coding: utf-8 -*-

"""Gene Substitutions.

Gene substitutions are legacy statements defined in BEL 1.0. BEL 2.0 recommends using HGVS strings. Luckily,
the information contained in a BEL 1.0 encoding, such as :code:`g(HGNC:APP,sub(G,275341,C))` can be
automatically translated to the appropriate HGVS :code:`g(HGNC:APP, var(c.275341G>C))`, assuming that all
substitutions are using the reference coding gene sequence for numbering and not the genomic reference.
The previous statements both produce the underlying data:

.. code-block:: python

    from pybel.constants import *

    {
        FUNCTION: GENE,
        NAMESPACE: 'HGNC',
        NAME: 'APP',
        VARIANTS: [
            {
                KIND: HGVS,
                IDENTIFIER: 'c.275341G>C',
            },
        ],
    }

.. seealso::

    - BEL 2.0 specification on `gene substitutions <http://openbel.org/language/version_2.0/bel_specification_version_2.0.html#_variants_2>`_
    - PyBEL module :py:class:`pybel.parser.modifiers.get_gene_substitution_language`
"""

import logging

from pyparsing import ParserElement, oneOf
from pyparsing import pyparsing_common as ppc

from ..utils import nest, one_of_tags
from ... import language
from ...constants import GSUB_POSITION, GSUB_REFERENCE, GSUB_VARIANT, HGVS, KIND

__all__ = [
    "get_gene_substitution_language",
]

logger = logging.getLogger(__name__)

dna_nucleotide = oneOf(list(language.dna_nucleotide_labels.keys()))
gsub_tag = one_of_tags(tags=["sub", "substitution"], canonical_tag=HGVS, name=KIND)


def get_gene_substitution_language() -> ParserElement:
    """Build a gene substitution parser."""
    parser_element = gsub_tag + nest(
        dna_nucleotide(GSUB_REFERENCE),
        ppc.integer(GSUB_POSITION),
        dna_nucleotide(GSUB_VARIANT),
    )
    parser_element.setParseAction(_handle_gsub)
    return parser_element


def _handle_gsub(line, _, tokens):
    upgraded = "c.{}{}>{}".format(tokens[GSUB_POSITION], tokens[GSUB_REFERENCE], tokens[GSUB_VARIANT])
    logger.debug("legacy sub() %s upgraded to %s", line, upgraded)
    tokens[HGVS] = upgraded
    del tokens[GSUB_POSITION]
    del tokens[GSUB_REFERENCE]
    del tokens[GSUB_VARIANT]
    return tokens



# -*- coding: utf-8 -*-

"""Test summary functions for edges."""

import unittest
from collections import Counter

from pybel import BELGraph
from pybel.dsl import protein
from pybel.examples import sialic_acid_graph
from pybel.language import Entity
from pybel.struct.summary.edge_summary import (
    count_annotations,
    get_annotation_values,
    get_annotation_values_by_annotation,
    get_annotations,
    get_unused_annotations,
    get_unused_list_annotation_values,
    iter_annotation_value_pairs,
    iter_annotation_values,
)
from pybel.testing.utils import n


class TestEdgeSummary(unittest.TestCase):
    """Test summary functions for edges."""

    def test_1(self):
        """Test iterating over annotation/value pairs."""
        graph = BELGraph()
        graph.annotation_list.update(
            {
                "A": set("1234"),
                "B": set("XYZ"),
                "C": set("abcde"),
            }
        )
        u = protein("HGNC", name="U")
        v = protein("HGNC", name="V")
        w = protein("HGNC", name="W")

        graph.add_increases(u, v, evidence=n(), citation=n(), annotations={"A": {"1", "2"}, "B": {"X"}})

        graph.add_increases(
            u,
            w,
            evidence=n(),
            citation=n(),
            annotations={
                "A": {"1", "3"},
                "C": {"a"},
            },
        )

        graph.add_increases(
            w,
            v,
            evidence=n(),
            citation=n(),
        )

        x = dict(Counter((key, entity.identifier) for key, entity in iter_annotation_value_pairs(graph)))

        self.assertEqual(
            {
                ("A", "1"): 2,
                ("A", "2"): 1,
                ("A", "3"): 1,
                ("B", "X"): 1,
                ("C", "a"): 1,
            },
            x,
        )

        y = Counter(iter_annotation_values(graph, "A"))
        self.assertEqual(x["A", "1"] + x["A", "2"] + x["A", "3"], sum(y.values()))

        y = Counter(iter_annotation_values(graph, "B"))
        self.assertEqual(x["B", "X"], sum(y.values()))

        y = Counter(iter_annotation_values(graph, "C"))
        self.assertEqual(x["C", "a"], sum(y.values()))

    def test_get_annotation_values(self):
        """Test getting annotation values."""
        expected = {
            "Confidence": {
                Entity(namespace="Confidence", identifier="High"),
                Entity(namespace="Confidence", identifier="Low"),
            },
            "Species": {
                Entity(namespace="Species", identifier="9606"),
            },
        }

        self.assertEqual({"Confidence", "Species"}, get_annotations(sialic_acid_graph))
        self.assertEqual({"Confidence": 8, "Species": 8}, dict(count_annotations(sialic_acid_graph)))

        annotation_values_by_annotation = get_annotation_values_by_annotation(sialic_acid_graph)
        self.assertEqual(expected, annotation_values_by_annotation)

        annotation_values = get_annotation_values(sialic_acid_graph, "Confidence")
        self.assertEqual(expected["Confidence"], annotation_values)

    def test_get_unused_annotation_url(self):
        graph = BELGraph()
        name = n()
        graph.annotation_url[name] = n()
        self.assertEqual({name}, get_unused_annotations(graph))

    def test_get_unused_annotation_pattern(self):
        graph = BELGraph()
        name = n()
        graph.annotation_pattern[name] = n()
        self.assertEqual({name}, get_unused_annotations(graph))

    def test_get_unused_annotation_list(self):
        graph = BELGraph()
        name = n()
        graph.annotation_pattern[name] = {n(), n(), n()}
        self.assertEqual({name}, get_unused_annotations(graph))

    def test_get_unused_annotation_list_values(self):
        """Test getting unused annotation list values."""
        graph = BELGraph()
        annotation_key = "test"
        graph.annotation_list[annotation_key] = set("abc")
        graph.add_increases(
            protein(n(), n()),
            protein(n(), n()),
            citation=n(),
            evidence=n(),
            annotations={annotation_key: {"a"}},
        )

        rv = get_unused_list_annotation_values(graph)
        self.assertIsInstance(rv, dict)

        self.assertEqual({annotation_key: set("bc")}, rv)



# -*- coding: utf-8 -*-

"""Grounding for PyBEL.

Why does this module exist, even though BEL relies on definitions of external vocabularies?

BEL namespaces only have names, and there's no standard for mapping back to identifiers.
PyBEL has a internal rule that for any given namespace, it will try and look up another
"identifier" space in the same directory. However, this is an implementation detail and does

This module uses PyOBO to look the identifiers for nodes in a BEL graph when possible
or replace the label given with BEP-0008 (OBO-style) syntax. It also normalizes namespace
names to their standards, as defined by Identifiers.org/OBOFoundry/PyOBO internal standard.

Finally, it has a tool that allows for the definition of remapping of namespace/name pairs.
Right now, it's only set up to use the FamPlex mappings, but it will be made more extensible
to help support the clean-up of curation efforts that created their own low-quality terminologies
(publicly accessible examples include the Selventa Large Corpus, Selventa Small Corpus, HemeKG,
covid19kg, and the Causal Biological Networks database).

After installation with ``pip install git+https://github.com/hemekg/hemekg.git``, it can be run with:

.. code-block:: python

    import hemekg
    heme_graph = hemekg.get_graph()

    import pybel.grounding
    pybel.grounding.ground(heme_graph)

After installation with ``pip install git+https://github.com/covid19kg/covid19kg.git``, it can be run with:

.. code-block:: python

    import covid19kg
    covid19_graph = covid19kg.get_graph()

    import pybel.grounding
    pybel.grounding.ground(covid19_graph)

After installation with ``pip install git+https://github.com/cthoyt/selventa-knowledge.git``, it can be run with:

.. code-block:: python

    import selventa_knowledge
    selventa_graph = selventa_knowledge.get_graph()

    import pybel.grounding
    pybel.grounding.ground(selventa_graph)
"""

import logging
from typing import Any, Collection, Mapping, Optional, Tuple, Union

import pyobo
from protmapper.uniprot_client import get_id_from_mnemonic, get_mnemonic
from pyobo.getters import NoBuild
from pyobo.identifier_utils import normalize_prefix
from pyobo.xrefdb.sources.famplex import get_remapping
from tqdm.autonotebook import tqdm

from pybel.constants import (
    ACTIVITY,
    ANNOTATIONS,
    CONCEPT,
    EFFECT,
    FROM_LOC,
    FUSION,
    GMOD,
    IDENTIFIER,
    KIND,
    LOCATION,
    MEMBERS,
    MODIFIER,
    NAME,
    NAMESPACE,
    PARTNER_3P,
    PARTNER_5P,
    PMOD,
    PRODUCTS,
    REACTANTS,
    SOURCE_MODIFIER,
    TARGET_MODIFIER,
    TO_LOC,
    TRANSLOCATION,
    VARIANTS,
)
from pybel.dsl import BaseConcept
from pybel.io import from_nodelink, to_nodelink
from pybel.language import (
    Entity,
    activity_mapping,
    compartment_mapping,
    gmod_mappings,
    pmod_mappings,
    text_location_labels,
)
from pybel.struct import BELGraph, get_annotations, get_namespaces, get_ungrounded_nodes

__all__ = [
    "ground",
    "ground_nodelink",
]

logger = logging.getLogger(__name__)
SKIP = {"ncbigene", "pubchem.compound", "chembl.compound"}
NO_NAMES = {"fplx", "eccode", "dbsnp", "smiles", "inchi", "inchikey"}

# TODO will get updated
#: A mapping of (prefix, name) pairs to (prefix, identifier, name) triples
_NAME_REMAPPING = get_remapping()

#: A mapping of (prefix, identifier) pairs to (prefix, identifier, name) triples
_ID_REMAPPING = {}


def _get_name_remapping(prefix: str, name: str) -> Union[Tuple[str, str, str], Tuple[None, None, None]]:
    if prefix.lower() in {"sfam", "scomp"} and ("bel", name) in _NAME_REMAPPING:
        return _NAME_REMAPPING["bel", name]
    return _NAME_REMAPPING.get((prefix, name), (None, None, None))


def _get_id_remapping(prefix: str, identifier: str) -> Union[Tuple[str, str, str], Tuple[None, None, None]]:
    return _ID_REMAPPING.get((prefix, identifier), (None, None, None))


def ground(
    graph: BELGraph,
    remove_ungrounded: bool = True,
    skip_namespaces: Optional[Collection[str]] = None,
) -> BELGraph:
    """Ground all entities in a BEL graph."""
    j = to_nodelink(graph)
    ground_nodelink(j, skip_namespaces=skip_namespaces)
    graph = from_nodelink(j)

    remove_unused_annotation_metadata(graph)

    if remove_ungrounded:
        ungrounded_nodes = {
            node
            for node in get_ungrounded_nodes(graph)
            if not isinstance(node, BaseConcept) or node.namespace not in NO_NAMES
        }
        graph.remove_nodes_from(ungrounded_nodes)
        graph.namespace_url.clear()
        graph.namespace_pattern.clear()
        graph.namespace_pattern.update({namespace: ".*" for namespace in get_namespaces(graph)})

        graph.annotation_url.clear()
        graph.annotation_pattern.clear()
        graph.annotation_list.clear()
        graph.annotation_pattern.update({annotation: ".*" for annotation in get_annotations(graph)})

    return graph


def remove_unused_annotation_metadata(graph) -> None:
    used_annotations = get_annotations(graph)

    unused_patterns = set(graph.annotation_pattern) - used_annotations
    for annotation in unused_patterns:
        logger.warning("deleting unused annotation pattern: %s", annotation)
        del graph.annotation_pattern[annotation]

    unused_urls = set(graph.annotation_pattern) - used_annotations
    for annotation in unused_urls:
        logger.warning("deleting unused annotation URL: %s", annotation)
        del graph.annotation_url[annotation]

    unused_lists = set(graph.annotation_list) - used_annotations
    for annotation in unused_lists:
        logger.warning("deleting unused annotation list: %s", annotation)
        del graph.annotation_list[annotation]


def ground_nodelink(graph_nodelink_dict, skip_namespaces: Optional[Collection[str]] = None) -> None:
    """Ground entities in a nodelink data structure."""
    name = graph_nodelink_dict.get("graph", {}).get("name", "graph")

    for data in tqdm(graph_nodelink_dict["links"], desc="grounding edges in {}".format(name)):
        _process_edge_side(data.get(SOURCE_MODIFIER), skip_namespaces=skip_namespaces)
        _process_edge_side(data.get(TARGET_MODIFIER), skip_namespaces=skip_namespaces)
        if ANNOTATIONS in data:
            _process_annotations(data, skip_namespaces=skip_namespaces)

    for node in tqdm(graph_nodelink_dict["nodes"], desc="grounding nodes in {}".format(name)):
        _process_node(node, skip_namespaces=skip_namespaces)


_BEL_ANNOTATION_PREFIX_MAP = {
    "MeSHDisease": "mesh",
    "MeSHAnatomy": "mesh",
    "CellStructure": "mesh",
    "Species": "ncbitaxon",
    "Disease": "doid",
    "Cell": "cl",
    "Anatomy": "uberon",
}
_BEL_ANNOTATION_PREFIX_CATEGORY_MAP = {
    "MeSHDisease": "Disease",
    "MeSHAnatomy": "Anatomy",
}

_UNHANDLED_ANNOTATION = set()
CATEGORY_BLACKLIST = {
    "TextLocation",
}


def _process_annotations(
    data,
    remove_ungrounded: bool = False,
    skip_namespaces: Optional[Collection[str]] = None,
) -> None:
    """Process the annotations in a PyBEL edge data dictionary."""
    cell_line_entities = data[ANNOTATIONS].get("CellLine")
    if cell_line_entities:
        ne = []
        for entity in cell_line_entities:
            if entity[NAMESPACE] == "CellLine":
                _namespaces = [
                    "efo",
                    # 'clo',  # FIXME implement CLO in PyOBO then uncomment
                ]
                g_prefix, g_identifier, g_name = pyobo.ground(_namespaces, entity[IDENTIFIER])
                if g_prefix and g_identifier:
                    ne.append(Entity(namespace=g_prefix, identifier=g_identifier, name=g_name))
                elif not remove_ungrounded:
                    logger.warning('could not ground CellLine: "%s"', entity[IDENTIFIER])
                    ne.append(entity)
        data[ANNOTATIONS]["CellLine"] = ne

    # fix text locations
    text_location = data[ANNOTATIONS].get("TextLocation")
    if text_location:
        data[ANNOTATIONS]["TextLocation"] = [
            text_location_labels.get(entity.identifier, entity) for entity in text_location
        ]

    # remap category names
    data[ANNOTATIONS] = {
        _BEL_ANNOTATION_PREFIX_CATEGORY_MAP.get(category, category): entities
        for category, entities in data[ANNOTATIONS].items()
    }
    # fix namespaces that were categories before
    for category, entities in data[ANNOTATIONS].items():
        if category in CATEGORY_BLACKLIST:
            continue

        ne = []
        for entity in entities:
            if not isinstance(entity, dict):
                raise TypeError(f"entity should be a dict. got: {entity}")
            nn = _BEL_ANNOTATION_PREFIX_MAP.get(entity[NAMESPACE])
            if nn is not None:
                entity[NAMESPACE] = nn

            _process_concept(concept=entity, skip_namespaces=skip_namespaces)
            ne.append(entity)
        data[ANNOTATIONS][category] = ne


def _process_edge_side(side_data, skip_namespaces: Optional[Collection[str]] = None) -> bool:
    """Process an edge JSON object, in place."""
    if side_data is None:
        return True

    modifier = side_data.get(MODIFIER)
    effect = side_data.get(EFFECT)

    if modifier == ACTIVITY and effect is not None:
        _process_concept(concept=effect, skip_namespaces=skip_namespaces)
    elif modifier == TRANSLOCATION and effect is not None:
        _process_concept(concept=effect[FROM_LOC], skip_namespaces=skip_namespaces)
        _process_concept(concept=effect[TO_LOC], skip_namespaces=skip_namespaces)

    location = side_data.get(LOCATION)
    if location is not None:
        _process_concept(concept=location, skip_namespaces=skip_namespaces)


_UNHANDLED_NAMESPACES = set()


def _process_node(node: Mapping[str, Any], skip_namespaces: Optional[Collection[str]] = None) -> bool:
    """Process a node JSON object, in place.

    :return: If all parts of the node were successfully grounded
    """
    success = True
    if CONCEPT in node:
        success = success and _process_concept(concept=node[CONCEPT], node=node, skip_namespaces=skip_namespaces)
    if VARIANTS in node:
        success = success and _process_list(node[VARIANTS], skip_namespaces=skip_namespaces)
    if MEMBERS in node:
        success = success and _process_list(node[MEMBERS], skip_namespaces=skip_namespaces)
    if REACTANTS in node:
        success = success and _process_list(node[REACTANTS], skip_namespaces=skip_namespaces)
    if PRODUCTS in node:
        success = success and _process_list(node[PRODUCTS], skip_namespaces=skip_namespaces)
    if FUSION in node:
        success = success and _process_fusion(node[FUSION], skip_namespaces=skip_namespaces)
    return success


def _process_concept(*, concept, node=None, skip_namespaces: Optional[Collection[str]] = None) -> bool:
    """Process a node JSON object."""
    namespace = concept[NAMESPACE]
    if namespace.lower() in {"text", "fixme"}:
        return False

    if skip_namespaces and namespace in skip_namespaces:
        return True

    prefix = normalize_prefix(namespace)
    if prefix is None:
        logger.warning('could not normalize namespace "%s" in concept "%s"', namespace, concept)
        return False

    concept[NAMESPACE] = prefix

    identifier = concept.get(IDENTIFIER)
    name = concept.get(NAME)
    if identifier:  # don't trust whatever was put for the name, even if it's available
        map_success = _handle_identifier_not_name(
            concept=concept,
            prefix=prefix,
            identifier=identifier,
            skip_namespaces=skip_namespaces,
        )
        if not map_success:  # just in case the name gets put in the identifier
            map_success = _handle_name_and_not_identifier(
                concept=concept,
                prefix=prefix,
                name=identifier,
                node=node,
                skip_namespaces=skip_namespaces,
            )
    else:
        map_success = _handle_name_and_not_identifier(
            concept=concept,
            prefix=prefix,
            name=name,
            node=node,
            skip_namespaces=skip_namespaces,
        )

    if not map_success:
        return False

    _remap_by_identifier(concept)
    return True


def _remap_by_identifier(concept) -> bool:
    identifier = concept.get(IDENTIFIER)
    if identifier is None:
        return False
    namespace = concept[NAMESPACE]
    logger.debug("attempting to remap %s:%s", namespace, identifier)
    remapped_prefix, remapped_identifier, remapped_name = _get_id_remapping(namespace, identifier)
    logger.debug(
        "remapping result %s:%s ! %s",
        remapped_prefix,
        remapped_identifier,
        remapped_name,
    )
    if remapped_prefix:
        concept[NAMESPACE] = remapped_prefix
        concept[IDENTIFIER] = remapped_identifier
        concept[NAME] = remapped_name
        return True
    return False


def _handle_identifier_not_name(
    *,
    concept,
    prefix,
    identifier,
    skip_namespaces: Optional[Collection[str]] = None,
) -> bool:
    # Some namespaces are just too much of a problem at the moment to look up
    if prefix in SKIP:
        return False
    if skip_namespaces and prefix in skip_namespaces:
        return True

    if prefix in NO_NAMES:
        concept[NAME] = concept[IDENTIFIER]
        return True

    if prefix == "uniprot":
        concept[NAME] = get_mnemonic(identifier)
        return True

    try:
        id_name_mapping = pyobo.api.names.get_id_name_mapping(prefix)
    except NoBuild:
        return False

    if id_name_mapping is None:
        logger.warning('could not get names for prefix "%s"', prefix)
        return False

    name = id_name_mapping.get(identifier)
    if name is None:
        logger.warning("could not get name for curie %s:%s", prefix, identifier)
        return False
    concept[NAME] = name

    return True


def _handle_name_and_not_identifier(
    *,
    concept,
    prefix,
    name,
    node=None,
    skip_namespaces: Optional[Collection[str]] = None,
) -> bool:
    remapped_prefix, remapped_identifier, remapped_name = _get_name_remapping(prefix, name)
    if remapped_prefix:
        concept[NAMESPACE] = remapped_prefix
        concept[IDENTIFIER] = remapped_identifier
        concept[NAME] = remapped_name
        return True

    # Some namespaces are just too much of a problem at the moment to look up
    if prefix in SKIP:
        return False
    if skip_namespaces and prefix in skip_namespaces:
        return True

    concept[NAMESPACE] = prefix
    if prefix in NO_NAMES:
        concept[IDENTIFIER] = name
        return True

    if prefix == "bel" and node is not None and KIND in node:
        kind = node[KIND]
        if kind == PMOD and name in pmod_mappings:
            # the 0th position xref is the preferred one (usually GO)
            _mapped = pmod_mappings[name]["xrefs"][0]
        elif kind == GMOD and name in gmod_mappings:
            _mapped = gmod_mappings[name]["xrefs"][0]
        else:
            raise ValueError(f"invalid kind: {kind}")
        concept[NAMESPACE] = _mapped[NAMESPACE]
        concept[IDENTIFIER] = _mapped[IDENTIFIER]
        concept[NAME] = _mapped[NAME]
        return True
    elif prefix == "bel" and name in activity_mapping:
        _mapped = activity_mapping[name]
        concept[NAMESPACE] = _mapped[NAMESPACE]
        concept[IDENTIFIER] = _mapped[IDENTIFIER]
        concept[NAME] = _mapped[NAME]
        return True
    elif prefix == "bel" and name in compartment_mapping:
        _mapped = compartment_mapping[name]
        concept[NAMESPACE] = _mapped[NAMESPACE]
        concept[IDENTIFIER] = _mapped[IDENTIFIER]
        concept[NAME] = _mapped[NAME]
        return True
    elif prefix == "bel":
        logger.warning('could not figure out how to map bel ! "%s"', name)
        return False

    if prefix == "uniprot":
        # assume identifier given as name
        identifier = get_id_from_mnemonic(name)
        if identifier is not None:
            concept[IDENTIFIER] = identifier
            return True

        mnemomic = get_mnemonic(name, web_fallback=False)
        if mnemomic is not None:
            concept[IDENTIFIER] = name
            concept[NAME] = mnemomic
            return True

        logger.warning('could not interpret uniprot name: "%s"', name)
        return False

    try:
        id_name_mapping = pyobo.api.names.get_name_id_mapping(prefix)
    except NoBuild as e:
        logger.warning("could not get namespace %s - %s", prefix, e)
        return False

    if id_name_mapping is None:
        logger.warning("unhandled namespace in %s ! %s", prefix, name)
        return False

    identifier = id_name_mapping.get(name)
    if identifier is None:
        logger.warning('could not find name "%s" in namespace "%s"', name, prefix)
        return False

    concept[IDENTIFIER] = identifier
    concept[NAME] = name
    return True


def _process_fusion(fusion, skip_namespaces: Optional[Collection[str]] = None) -> bool:
    success_3p = _process_node(fusion[PARTNER_3P], skip_namespaces=skip_namespaces)
    success_5p = _process_node(fusion[PARTNER_5P], skip_namespaces=skip_namespaces)
    return success_3p and success_5p


def _process_list(members, skip_namespaces: Optional[Collection[str]] = None) -> bool:
    success = True
    for member in members:
        success = success and _process_node(member, skip_namespaces=skip_namespaces)
    return success



# -*- coding: utf-8 -*-

"""Convert the AD graph for Hipathia."""

import os

import click
from pyobo.cli_utils import verbose_option

import pybel
import pybel.grounding
from pybel.struct import get_subgraphs_by_annotation

HERE = os.path.dirname(__file__)
OUTPUT = os.path.join(HERE, 'output')
os.makedirs(OUTPUT, exist_ok=True)
PATH = os.path.join(HERE, 'alzheimers.bel.nodelink.json')


@click.command()
@verbose_option
def main():
    """Convert the AD graph to Hipathia."""
    graph = pybel.load(PATH)
    graph = pybel.grounding.ground(graph)

    graphs = get_subgraphs_by_annotation(graph, annotation='Subgraph')
    for name, graph in graphs.items():
        graph.name = name
        pybel.to_hipathia(graph, OUTPUT)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

"""An example describing statins."""

from ..dsl import Abundance, Protein
from ..resources import CHEBI_URL, CONFIDENCE_URL, EC_URL, HGNC_URL
from ..struct.graph import BELGraph

__all__ = [
    "statin_graph",
]

statin_graph = BELGraph(
    name="Statin Graph",
    version="1.0.1",
    description="The effects of statins from ChEBI",
    authors="Charles Tapley Hoyt",
    contact="cthoyt@gmail.com",
)

statin_graph.namespace_url.update(
    {
        "HGNC": HGNC_URL,
        "CHEBI": CHEBI_URL,
        "EC": EC_URL,
    }
)

statin_graph.annotation_url.update(
    {
        "Confidence": CONFIDENCE_URL,
    }
)

fluvastatin = Abundance(namespace="CHEBI", name="fluvastatin", identifier="38561")
avorastatin = Abundance(namespace="CHEBI", name="atorvastatin", identifier="39548")
synthetic_statin = Abundance(namespace="CHEBI", name="statin (synthetic)", identifier="87635")
statin = Abundance(namespace="CHEBI", name="statin", identifier="87631")
mevinolinic_acid = Abundance(namespace="CHEBI", name="mevinolinic acid", identifier="82985")
hmgcr_inhibitor = Abundance(
    namespace="CHEBI",
    identifier="35664",
    name="EC 1.1.1.34/EC 1.1.1.88 (hydroxymethylglutaryl-CoA reductase) inhibitor",
)
ec_11134 = Protein(namespace="EC", name="1.1.1.34")
ec_11188 = Protein(namespace="EC", name="1.1.1.88")

hmgcr = Protein(namespace="HGNC", name="HMGCR", identifier="5006")

statin_graph.add_is_a(avorastatin, synthetic_statin)
statin_graph.add_is_a(fluvastatin, synthetic_statin)
statin_graph.add_is_a(synthetic_statin, statin)
statin_graph.add_is_a(statin, hmgcr_inhibitor)
statin_graph.add_is_a(mevinolinic_acid, hmgcr_inhibitor)

statin_graph.add_is_a(hmgcr, ec_11134)

statin_graph.add_inhibits(
    hmgcr_inhibitor,
    ec_11134,
    evidence="From ChEBI",
    citation="23180789",
    annotations={
        "Confidence": "Axiomatic",
    },
)

statin_graph.add_inhibits(
    hmgcr_inhibitor,
    ec_11188,
    evidence="From ChEBI",
    citation="23180789",
    annotations={
        "Confidence": "Axiomatic",
    },
)



# -*- coding: utf-8 -*-

"""Functions to modify the metadata of graphs, their edges, and their nodes."""

import logging

from ..graph import BELGraph
from ..pipeline import in_place_transformation
from ...constants import ANNOTATIONS, CITATION, IDENTIFIER, NAMESPACE

__all__ = [
    "strip_annotations",
    "add_annotation_value",
    "remove_annotation_value",
    "remove_extra_citation_metadata",
]

logger = logging.getLogger(__name__)


@in_place_transformation
def strip_annotations(graph: BELGraph) -> None:
    """Strip all the annotations from a BEL graph.

    :param graph: A BEL graph
    """
    for u, v, k in graph.edges(keys=True):
        if ANNOTATIONS in graph[u][v][k]:
            del graph[u][v][k][ANNOTATIONS]


@in_place_transformation
def add_annotation_value(graph: BELGraph, annotation: str, value: str, strict: bool = True) -> None:
    """Add the given annotation/value pair to all qualified edges.

    :param graph: A BEL graph
    :param annotation:
    :param value:
    :param strict: Should the function ensure the annotation has already been defined?
    """
    if strict and annotation not in graph.defined_annotation_keywords:
        raise ValueError("annotation not defined: {}".format(annotation))

    for u, v, k in graph.edges(keys=True):
        if ANNOTATIONS not in graph[u][v][k]:
            continue

        if annotation not in graph[u][v][k][ANNOTATIONS]:
            graph[u][v][k][ANNOTATIONS][annotation] = {}

        graph[u][v][k][ANNOTATIONS][annotation][value] = True


@in_place_transformation
def remove_annotation_value(graph: BELGraph, annotation: str, value: str) -> None:
    """Remove the given annotation/value pair to all qualified edges.

    :param graph: A BEL graph
    :param annotation:
    :param value:
    """
    if annotation not in graph.defined_annotation_keywords:
        logger.warning("annotation was not defined: %s", annotation)
        return

    for u, v, k in graph.edges(keys=True):
        if ANNOTATIONS not in graph[u][v][k]:
            continue

        if annotation not in graph[u][v][k][ANNOTATIONS]:
            continue

        if value not in graph[u][v][k][ANNOTATIONS][annotation]:
            continue

        del graph[u][v][k][ANNOTATIONS][annotation][value]


_CITATION_KEEP_KEYS = {IDENTIFIER, NAMESPACE}


@in_place_transformation
def remove_extra_citation_metadata(graph) -> None:
    """Remove superfluous metadata associated with a citation (that isn't the db/id).

    Best practice is to add this information programmatically.
    """
    for u, v, k in graph.edges(keys=True):
        if CITATION not in graph[u][v][k]:
            continue
        for key in list(graph[u][v][k][CITATION]):
            if key not in _CITATION_KEEP_KEYS:
                del graph[u][v][k][CITATION][key]



# -*- coding: utf-8 -*-

"""Support for displaying BEL graphs in Jupyter notebooks."""

from .inline import to_jupyter, to_jupyter_str
from .visualization import to_html, to_html_file

__all__ = [
    "to_html",
    "to_html_file",
    "to_jupyter",
    "to_jupyter_str",
]



# -*- coding: utf-8 -*-

"""Tests for the umbrella node-link JSON exporter."""

import unittest

from pybel.io.umbrella_nodelink import to_umbrella_nodelink
from tests.test_io.test_cx.examples import example_graph


class TestUmbrellaNodeLinkExporter(unittest.TestCase):
    """Tests for the umbrella node-link JSON exporter."""

    def test_exporter_new_nodes(self):
        """Test new nodes created."""
        # Check original number of nodes and edges in the example BEL Graph
        self.assertEqual(29, example_graph.number_of_nodes())
        self.assertEqual(32, example_graph.number_of_edges())

        custom_json_dict = to_umbrella_nodelink(example_graph)

        self.assertEqual(32, len(custom_json_dict["nodes"]))

        # 3 new nodes are created:
        self.assertIn(
            'act(p(hgnc:MAPK1), ma(go:0016301 ! "kinase activity"))',
            custom_json_dict["nodes"],
        )
        self.assertIn(
            'act(p(hgnc:PTK2, pmod(go:0006468 ! "protein phosphorylation", Tyr, 925)), ma(go:0016301 ! "kinase activity"))',
            custom_json_dict["nodes"],
        )
        self.assertIn(
            'act(p(fus(hgnc:BCR, "?", hgnc:ABL1, "?")), ma(go:0016301 ! "kinase activity"))',
            custom_json_dict["nodes"],
        )

    def test_exporter_edges(self):
        """Test no new edges created."""
        # Check original number of nodes and edges in the example BEL Graph
        self.assertEqual(29, example_graph.number_of_nodes(), msg="Wrong number of nodes")
        self.assertEqual(32, example_graph.number_of_edges(), msg="Wrong number of edges")

        custom_json_dict = to_umbrella_nodelink(example_graph)

        # Number of edges is maintained
        self.assertEqual(
            32,
            len(custom_json_dict["links"]),
            msg="Wrong number of links in Umbrella JSON",
        )



# -*- coding: utf-8 -*-

"""Pre-defined predicates for nodes."""

from ..utils import part_has_modifier
from ...graph import BELGraph
from ....constants import (
    ACTIVITY,
    DEGRADATION,
    SOURCE_MODIFIER,
    TARGET_MODIFIER,
    TRANSLOCATION,
)
from ....dsl import BaseEntity

__all__ = [
    "has_edge_modifier",
    "has_activity",
    "is_degraded",
    "is_translocated",
]


def has_edge_modifier(graph: BELGraph, node: BaseEntity, modifier: str) -> bool:
    """Return true if over any of a nodes edges, it has a given modifier.

     Modifier can be one of:

     - :data:`pybel.constants.ACTIVITY`,
     - :data:`pybel.constants.DEGRADATION`
     - :data:`pybel.constants.TRANSLOCATION`.

    :param modifier: One of :data:`pybel.constants.ACTIVITY`, :data:`pybel.constants.DEGRADATION`, or
                        :data:`pybel.constants.TRANSLOCATION`
    """
    modifier_in_subject = any(
        part_has_modifier(d, SOURCE_MODIFIER, modifier) for _, _, d in graph.out_edges(node, data=True)
    )

    modifier_in_object = any(
        part_has_modifier(d, TARGET_MODIFIER, modifier) for _, _, d in graph.in_edges(node, data=True)
    )

    return modifier_in_subject or modifier_in_object


def has_activity(graph: BELGraph, node: BaseEntity) -> bool:
    """Return true if over any of the node's edges, it has a molecular activity."""
    return has_edge_modifier(graph, node, ACTIVITY)


def is_degraded(graph: BELGraph, node: BaseEntity) -> bool:
    """Return true if over any of the node's edges, it is degraded."""
    return has_edge_modifier(graph, node, DEGRADATION)


def is_translocated(graph: BELGraph, node: BaseEntity) -> bool:
    """Return true if over any of the node's edges, it is translocated."""
    return has_edge_modifier(graph, node, TRANSLOCATION)



# -*- coding: utf-8 -*-

"""Make hetionet exports."""

import os
from random import choice

import click
import networkx as nx
import pandas as pd
from tqdm.autonotebook import tqdm

from pybel import (
    from_nodelink_gz,
    get_hetionet,
    to_bel_script,
    to_bel_script_gz,
    to_graphdati_file,
    to_graphdati_gz,
    to_graphdati_jsonl_gz,
    to_nodelink_gz,
)
from pybel.canonicalize import edge_to_bel
from pybel.struct.summary.edge_summary import get_metaedge_to_key


@click.command()
@click.option(
    "--directory",
    default=os.getcwd(),
    required=True,
    show_default=True,
    type=click.Path(dir_okay=True),
)
def main(directory: str):
    """Make hetionet exports."""
    path = os.path.join(directory, "hetionet.bel.nodelink.json.gz")
    if not os.path.exists(path):
        graph = get_hetionet()
        to_nodelink_gz(graph, path)
    else:
        click.echo("loading pickle from {}".format(path))
        graph = from_nodelink_gz(path)

    output_bel_gz_path = os.path.join(directory, "hetionet.bel.gz")
    if not os.path.exists(output_bel_gz_path):
        click.echo("outputting whole hetionet as BEL GZ to {}".format(output_bel_gz_path))
        to_bel_script_gz(graph, output_bel_gz_path, use_identifiers=True)

    output_graphdati_jsonl_gz_path = os.path.join(directory, "hetionet.bel.graphdati.jsonl.gz")
    if not os.path.exists(output_graphdati_jsonl_gz_path):
        click.echo("outputting whole hetionet as BEL GraphDati JSONL GZ to {}".format(output_graphdati_jsonl_gz_path))
        to_graphdati_jsonl_gz(graph, output_graphdati_jsonl_gz_path, use_identifiers=True)

    output_graphdati_gz_path = os.path.join(directory, "hetionet.bel.graphdati.json.gz")
    if not os.path.exists(output_graphdati_gz_path):
        click.echo("outputting whole hetionet as BEL GraphDati JSON GZ to {}".format(output_graphdati_gz_path))
        to_graphdati_gz(graph, output_graphdati_gz_path, use_identifiers=True)

    summary_tsv_path = os.path.join(directory, "hetionet_summary.tsv")
    if not os.path.exists(summary_tsv_path):
        click.echo("getting metaedges")
        rows = []
        keep_keys = set()
        for value in get_metaedge_to_key(graph).values():
            u, v, key = choice(list(value))
            keep_keys.add(key)
            d = graph[u][v][key]
            bel = edge_to_bel(u, v, d, use_identifiers=True)
            rows.append((key[:8], bel))

        df = pd.DataFrame(rows, columns=["key", "bel"])
        df.to_csv(summary_tsv_path, sep="\t", index=False)

        non_sample_edges = [
            (u, v, k, d)
            for u, v, k, d in tqdm(
                graph.edges(keys=True, data=True),
                desc="Getting non-sample edges to remove",
            )
            if k not in keep_keys
        ]
        click.echo("Removing non-sample edges")
        graph.remove_edges_from(non_sample_edges)
        graph.remove_nodes_from(list(nx.isolates(graph)))

        sample_bel_path = os.path.join(directory, "hetionet_sample.bel")
        click.echo("outputting sample hetionet in BEL to {}".format(sample_bel_path))
        to_bel_script(graph, sample_bel_path, use_identifiers=True)

        sample_graphdati_path = os.path.join(directory, "hetionet_sample.bel.graphdati.json")
        click.echo("outputting sample hetionet in BEL to {}".format(sample_bel_path))
        to_graphdati_file(graph, sample_graphdati_path, use_identifiers=True, indent=2)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

"""Summary functions for edges in BEL graphs."""

from collections import Counter, defaultdict
from random import choice
from typing import Iterable, Mapping, Set, Tuple

from ..filters.edge_predicates import edge_has_annotation
from ..graph import BELGraph
from ...canonicalize import edge_to_bel
from ...constants import ANNOTATIONS, RELATION
from ...dsl import BaseEntity
from ...language import Entity
from ...utils import CanonicalEdge, canonicalize_edge

__all__ = [
    "iter_annotation_value_pairs",
    "iter_annotation_values",
    "get_annotation_values_by_annotation",
    "get_annotation_values",
    "count_relations",
    "get_annotations",
    "count_annotations",
    "get_unused_annotations",
    "get_unused_list_annotation_values",
    "get_metaedge_to_key",
    "iter_sample_metaedges",
]


def iter_annotation_value_pairs(graph: BELGraph) -> Iterable[Tuple[str, Entity]]:
    """Iterate over the key/value pairs, with duplicates, for each annotation used in a BEL graph.

    :param graph: A BEL graph
    """
    return (
        (key, entity)
        for _, _, data in graph.edges(data=True)
        for key, entities in data.get(ANNOTATIONS, {}).items()
        for entity in entities
    )


def iter_annotation_values(graph: BELGraph, annotation: str) -> Iterable[Entity]:
    """Iterate over all of the values for an annotation used in the graph.

    :param graph: A BEL graph
    :param annotation: The annotation to grab
    """
    return (
        entity
        for _, _, data in graph.edges(data=True)
        if edge_has_annotation(data, annotation)
        for entity in data[ANNOTATIONS][annotation]
    )


def _group_dict_set(iterator):
    """Make a dict that accumulates the values for each key in an iterator of doubles.

    :param iter[tuple[A,B]] iterator: An iterator
    :rtype: dict[A,set[B]]
    """
    d = defaultdict(set)
    for key, value in iterator:
        d[key].add(value)
    return dict(d)


def get_annotation_values_by_annotation(graph: BELGraph) -> Mapping[str, Set[Entity]]:
    """Get the set of values for each annotation used in a BEL graph.

    :param graph: A BEL graph
    :return: A dictionary of {annotation key: set of annotation values}
    """
    return _group_dict_set(iter_annotation_value_pairs(graph))


def get_annotation_values(graph: BELGraph, annotation: str) -> Set[Entity]:
    """Get all values for the given annotation.

    :param graph: A BEL graph
    :param annotation: The annotation to summarize
    :return: A set of all annotation values
    """
    return set(iter_annotation_values(graph, annotation))


def count_relations(graph: BELGraph) -> Counter:
    """Return a histogram over all relationships in a graph.

    :param graph: A BEL graph
    :return: A Counter from {relation type: frequency}
    """
    return Counter(data[RELATION] for _, _, data in graph.edges(data=True))


def get_unused_annotations(graph: BELGraph) -> Set[str]:
    """Get the set of all annotations that are defined in a graph, but are never used.

    :param graph: A BEL graph
    :return: A set of annotations
    """
    return graph.defined_annotation_keywords - get_annotations(graph)


def get_annotations(graph: BELGraph) -> Set[str]:
    """Get the set of annotations used in the graph.

    :param graph: A BEL graph
    :return: A set of annotation keys
    """
    return set(_annotation_iter_helper(graph))


def count_annotations(graph: BELGraph) -> Counter:
    """Count how many times each annotation is used in the graph.

    :param graph: A BEL graph
    :return: A Counter from {annotation key: frequency}
    """
    return Counter(_annotation_iter_helper(graph))


def _annotation_iter_helper(graph: BELGraph) -> Iterable[str]:
    """Iterate over the annotation keys.

    :param graph: A BEL graph
    """
    return (key for _, _, data in graph.edges(data=True) if ANNOTATIONS in data for key in data[ANNOTATIONS])


def get_unused_list_annotation_values(graph: BELGraph) -> Mapping[str, Set[str]]:
    """Get all of the unused values for list annotations.

    :param graph: A BEL graph
    :return: A dictionary of {str annotation: set of str values that aren't used}
    """
    result = {}
    for annotation, values in graph.annotation_list.items():
        unused = values - {e.identifier for e in get_annotation_values(graph, annotation)}
        if unused:
            result[annotation] = unused
    return result


def get_metaedge_to_key(
    graph: BELGraph,
) -> Mapping[CanonicalEdge, Set[Tuple[BaseEntity, BaseEntity, str]]]:
    """Get all edge types."""
    rv = defaultdict(set)
    for u, v, k, d in graph.edges(keys=True, data=True):
        rel, u_mod, v_mod = canonicalize_edge(d)
        rv[u.__class__.__name__, u_mod, rel, v.__class__.__name__, v_mod].add((u, v, k))
    return dict(rv)


def iter_sample_metaedges(graph: BELGraph):
    """Iterate sampled metaedges."""
    for k, value in get_metaedge_to_key(graph).items():
        u, v, key = choice(list(value))
        d = graph[u][v][key]
        bel = edge_to_bel(u, v, d, use_identifiers=True)
        yield (u, v, key, d, *k, bel)



# -*- coding: utf-8 -*-

"""Mocks for PyBEL testing."""

from typing import Iterable

from ..manager.models import Network
from ..struct import union


class MockQueryManager:
    """A mock manager."""

    def __init__(self, graphs=None):
        """Build a mock manager appropriate for testing the pipeline and query builders.

        :param Optional[list[pybel.BELGraph]] graphs: A list of BEL graphs to index
        """
        self.graphs = []

        #: A lookup for nodes from the node hash (string) to the node tuple
        self.hash_to_node = {}

        #: A lookup from network identifier to graph
        self.id_graph = {}

        if graphs is not None:
            for graph in graphs:
                self.insert_graph(graph)

    def count_networks(self) -> int:
        """Count networks in the manager."""
        return len(self.graphs)

    def insert_graph(self, graph) -> Network:
        """Insert a graph and ensure its nodes are cached.

        :param pybel.BELGraph graph:
        """
        network_id = len(self.graphs)
        self.graphs.append(graph)
        self.id_graph[network_id] = graph

        for node in graph:
            self.hash_to_node[node.md5] = node

        return Network(id=network_id)

    def get_graph_by_ids(self, network_ids: Iterable[int]):
        """Get a graph from the union of multiple networks.

        :param network_ids: The identifiers of networks in the database
        :rtype: pybel.BELGraph
        """
        network_ids = list(network_ids)

        if len(network_ids) == 1:
            return self.id_graph[network_ids[0]]

        graphs = [self.id_graph[graph_id] for graph_id in network_ids]

        return union(graphs)

    def get_dsl_by_hash(self, md5: str):
        """Get a DSL by its hash.

        :rtype: Optional[BaseEntity]
        """
        return self.hash_to_node.get(md5)



# -*- coding: utf-8 -*-

import re
import unittest

from pybel.constants import DIRTY
from pybel.exceptions import MissingNamespaceRegexWarning, NakedNameWarning
from pybel.parser import ConceptParser


class _ParserMixin(unittest.TestCase):
    def setUp(self):
        self.namespace_to_term = {
            "A": {
                (None, "1"): "P",
                (None, "2"): "P",
                (None, "3"): "P",
            },
            "B": {
                (None, "4"): "P",
                (None, "5"): "P",
                (None, "6"): "P",
            },
        }


class TestConceptEnumerated(_ParserMixin):
    def setUp(self):
        super().setUp()
        self.parser = ConceptParser(namespace_to_term_to_encoding=self.namespace_to_term)

    def test_valid_1(self):
        s = "A:3"
        result = self.parser.parseString(s)

        self.assertIn("namespace", result)
        self.assertIn("name", result)
        self.assertEqual("A", result["namespace"])
        self.assertEqual("3", result["name"])

    def test_valid_2(self):
        s = 'A:"3"'
        result = self.parser.parseString(s)

        self.assertIn("namespace", result)
        self.assertIn("name", result)
        self.assertEqual("A", result["namespace"])
        self.assertEqual("3", result["name"])

    def test_invalid_1(self):
        s = "C:4"
        with self.assertRaises(Exception):
            self.parser.parseString(s)

    def test_invalid_2(self):
        s = "A:4"
        with self.assertRaises(Exception):
            self.parser.parseString(s)

    def test_invalid_3(self):
        s = "bare"
        with self.assertRaises(NakedNameWarning):
            self.parser.parseString(s)

    def test_invalid_4(self):
        s = '"quoted"'
        with self.assertRaises(NakedNameWarning):
            self.parser.parseString(s)


class TestConceptParserDefault(_ParserMixin):
    """Tests where the concept parser allows an enumerated list of unqualified entities."""

    def setUp(self):
        super().setUp()
        default_namespace = {"X", "Y", "W Z"}
        self.parser = ConceptParser(
            namespace_to_term_to_encoding=self.namespace_to_term,
            default_namespace=default_namespace,
        )

    def test_valid_1(self):
        s = "A:3"
        result = self.parser.parseString(s)

        self.assertIn("namespace", result)
        self.assertIn("name", result)
        self.assertNotIn("identifier", result)
        self.assertEqual("A", result["namespace"])
        self.assertEqual("3", result["name"])

    def test_valid_2(self):
        s = "X"
        result = self.parser.parseString(s)

        self.assertIn("name", result)
        self.assertEqual("X", result["name"])

    def test_valid_3(self):
        s = '"W Z"'
        result = self.parser.parseString(s)

        self.assertIn("name", result)
        self.assertNotIn("identifier", result)
        self.assertEqual("W Z", result["name"])

    def test_not_in_defaultNs(self):
        s = "D"
        with self.assertRaises(Exception):
            self.parser.parseString(s)


class TestConceptParserLenient(_ParserMixin):
    """Tests where naked names are allowed."""

    def setUp(self):
        super().setUp()
        self.parser = ConceptParser(
            namespace_to_term_to_encoding=self.namespace_to_term,
            allow_naked_names=True,
        )

    def test_valid_1(self):
        s = "A:3"
        result = self.parser.parseString(s)

        self.assertIn("namespace", result)
        self.assertIn("name", result)
        self.assertNotIn("identifier", result)
        self.assertEqual("A", result["namespace"])
        self.assertEqual("3", result["name"])

    def test_valid_2(self):
        s = 'A:"3"'
        result = self.parser.parseString(s)

        self.assertIn("namespace", result)
        self.assertIn("name", result)
        self.assertNotIn("identifier", result)
        self.assertEqual("A", result["namespace"])
        self.assertEqual("3", result["name"])

    def test_invalid_1(self):
        s = "C:4"
        with self.assertRaises(Exception):
            self.parser.parseString(s)

    def test_invalid_2(self):
        s = "A:4"
        with self.assertRaises(Exception):
            self.parser.parseString(s)

    def test_not_invalid_3(self):
        s = "bare"
        result = self.parser.parseString(s)

        self.assertIn("namespace", result)
        self.assertIn("name", result)
        self.assertNotIn("identifier", result)
        self.assertEqual(DIRTY, result["namespace"])
        self.assertEqual("bare", result["name"])

    def test_not_invalid_4(self):
        s = '"quoted"'
        result = self.parser.parseString(s)

        self.assertIn("namespace", result)
        self.assertIn("name", result)
        self.assertNotIn("identifier", result)
        self.assertEqual(DIRTY, result["namespace"])
        self.assertEqual("quoted", result["name"])


class TestConceptParserRegex(unittest.TestCase):
    """Tests for regular expression parsing"""

    def setUp(self) -> None:
        self.parser = ConceptParser(
            namespace_to_pattern={
                "hgnc": re.compile(r"\d+"),
                "ec-code": re.compile(r".+"),
            }
        )
        self.assertEqual({}, self.parser.namespace_to_identifier_to_encoding)
        self.assertEqual({}, self.parser.namespace_to_name_to_encoding)

    def test_valid(self):
        for curie, namespace, name in [
            ("hgnc:391", "hgnc", "391"),
            ("ec-code:1.1.1.27", "ec-code", "1.1.1.27"),
        ]:
            with self.subTest(curie=curie):
                result = self.parser.parseString(curie)
                self.assertIn("namespace", result)
                self.assertIn("name", result)
                self.assertNotIn("identifier", result)
                self.assertEqual(namespace, result["namespace"])
                self.assertEqual(name, result["name"])

    def test_invalid(self):
        """Test invalid BEL term."""
        s = "hgnc:AKT1"
        with self.assertRaises(MissingNamespaceRegexWarning):
            result = self.parser.parseString(s)
            print(result.asDict())

    def test_valid_obo(self):
        """Test parsing an identifier that has a name."""
        s = "hgnc:391 ! AKT1"
        result = self.parser.parseString(s)

        self.assertIn("namespace", result)
        self.assertIn("name", result)
        self.assertIn("identifier", result)
        self.assertEqual("hgnc", result["namespace"])
        self.assertEqual("AKT1", result["name"])
        self.assertEqual("391", result["identifier"])

    def test_invalid_obo(self):
        """Test parsing an OBO-style identifier where the identifier and name are switched."""
        s = "hgnc:AKT1 ! 391"
        with self.assertRaises(MissingNamespaceRegexWarning):
            result = self.parser.parseString(s)
            print(result.asDict())



# -*- coding: utf-8 -*-

"""Testing resources for PyBEL."""

import os

__all__ = [
    # BELNS
    "test_ns_1",
    "test_ns_2",
    "test_ns_empty",
    # BELANNO
    "test_an_1",
    # BEL
    "test_bel_simple",
    "test_bel_slushy",
    "test_bel_thorough",
    "test_bel_isolated",
    "test_bel_misordered",
    "test_bel_no_identifier_valiation",
    # JSON
    "test_jgif_path",
]

HERE = os.path.dirname(os.path.realpath(__file__))

resources_dir = os.path.join(HERE, "resources")

# BELNS Files
belns_dir_path = os.path.join(resources_dir, "belns")

test_ns_1 = os.path.join(belns_dir_path, "test_ns_1.belns")
test_ns_2 = os.path.join(belns_dir_path, "test_ns_1_updated.belns")
test_ns_empty = os.path.join(belns_dir_path, "test_ns_empty.belns")

# BELANNO Files
belanno_dir_path = os.path.join(resources_dir, "belanno")

test_an_1 = os.path.join(belanno_dir_path, "test_an_1.belanno")

# BEL Files
bel_dir_path = os.path.join(resources_dir, "bel")

test_bel_simple = os.path.join(bel_dir_path, "test_bel.bel")
test_bel_slushy = os.path.join(bel_dir_path, "slushy.bel")
test_bel_thorough = os.path.join(bel_dir_path, "thorough.bel")
test_bel_isolated = os.path.join(bel_dir_path, "isolated.bel")
test_bel_misordered = os.path.join(bel_dir_path, "misordered.bel")
test_bel_no_identifier_valiation = os.path.join(bel_dir_path, "no_identifier_validation_test.bel")
test_bel_with_obo = os.path.join(bel_dir_path, "obo.bel")

# JSON Files

test_jgif_path = os.path.join(bel_dir_path, "Cytotoxic T-cell Signaling-2.0-Hs.json")



# -*- coding: utf-8 -*-

"""Induction methods for graphs over shortest paths."""

import itertools as itt
import logging
import random
from typing import Iterable, List, Optional, Set, Tuple

import networkx as nx

from .utils import get_subgraph_by_induction
from ...graph import BELGraph
from ...pipeline import transformation
from ....constants import FUNCTION, PATHOLOGY
from ....dsl import BaseEntity

__all__ = [
    "get_nodes_in_all_shortest_paths",
    "get_subgraph_by_all_shortest_paths",
    "get_random_path",
]

logger = logging.getLogger(__name__)


def _remove_pathologies_oop(graph: BELGraph):
    """Remove pathology nodes from the graph."""
    rv = graph.copy()
    victims = [node for node in rv if node[FUNCTION] == PATHOLOGY]
    rv.remove_nodes_from(victims)
    return rv


def _iterate_nodes_in_shortest_paths(
    graph: BELGraph,
    nodes: Iterable[BaseEntity],
    weight: Optional[str] = None,
) -> Iterable[BaseEntity]:
    """Iterate over nodes in the shortest paths between all pairs of nodes in the given list."""
    for source, target in itt.product(nodes, repeat=2):
        try:
            paths = nx.all_shortest_paths(graph, source, target, weight=weight)
            for path in paths:
                for node in path:
                    yield node
        except nx.exception.NetworkXNoPath:
            continue


def get_nodes_in_all_shortest_paths(
    graph: BELGraph,
    nodes: Iterable[BaseEntity],
    weight: Optional[str] = None,
    remove_pathologies: bool = False,
) -> Set[BaseEntity]:
    """Get a set of nodes in all shortest paths between the given nodes.

    Thinly wraps :func:`networkx.all_shortest_paths`.

    :param graph: A BEL graph
    :param nodes: The list of nodes to use to use to find all shortest paths
    :param weight: Edge data key corresponding to the edge weight. If none, uses unweighted search.
    :param remove_pathologies: Should pathology nodes be removed first?
    :return: A set of nodes appearing in the shortest paths between nodes in the BEL graph

    .. note:: This can be trivially parallelized using :func:`networkx.single_source_shortest_path`
    """
    if remove_pathologies:
        graph = _remove_pathologies_oop(graph)

    return set(_iterate_nodes_in_shortest_paths(graph, nodes, weight=weight))


@transformation
def get_subgraph_by_all_shortest_paths(
    graph,
    nodes: Iterable[BaseEntity],
    weight: Optional[str] = None,
    remove_pathologies: bool = False,
) -> Optional[BELGraph]:
    """Induce a subgraph over the nodes in the pairwise shortest paths between all of the nodes in the given list.

    :param pybel.BELGraph graph: A BEL graph
    :param nodes: A set of nodes over which to calculate shortest paths
    :param weight: Edge data key corresponding to the edge weight. If None, performs unweighted search
    :param remove_pathologies: Should the pathology nodes be deleted before getting shortest paths?
    :return: A BEL graph induced over the nodes appearing in the shortest paths between the given nodes
    :rtype: Optional[pybel.BELGraph]
    """
    query_nodes = []

    for node in nodes:
        if node not in graph:
            logger.debug("%s not in %s", node, graph)
            continue
        query_nodes.append(node)

    if not query_nodes:
        return

    induced_nodes = get_nodes_in_all_shortest_paths(
        graph,
        query_nodes,
        weight=weight,
        remove_pathologies=remove_pathologies,
    )

    if not induced_nodes:
        return

    return get_subgraph_by_induction(graph, induced_nodes)


def get_random_path(graph: BELGraph) -> List[BaseEntity]:
    """Get a random path from the graph as a list of nodes.

    :param graph: A BEL graph
    """
    wg = graph.to_undirected()

    nodes = wg.nodes()

    def pick_random_pair() -> Tuple[BaseEntity, BaseEntity]:
        """Get a pair of random nodes."""
        return random.sample(nodes, k=2)

    source, target = pick_random_pair()

    tries = 0
    sentinel_tries = 5

    while not nx.has_path(wg, source, target) and tries < sentinel_tries:
        tries += 1
        source, target = pick_random_pair()

    if tries == sentinel_tries:
        return [source]

    return nx.shortest_path(wg, source=source, target=target)



# -*- coding: utf-8 -*-

"""Filter functions for nodes in BEL graphs.

A node predicate is a function that takes two arguments: a :class:`BELGraph` and a node. It returns a boolean
representing whether the node passed the given test.

This module contains a set of default functions for filtering lists of nodes and building node predicates.

A general use for a node predicate is to use the built-in :func:`filter` in code like
:code:`filter(your_node_predicate, graph)`
"""

from typing import Iterable, Set

from .node_predicate_builders import (
    function_inclusion_filter_builder,
    namespace_inclusion_builder,
)
from .node_predicates import concatenate_node_predicates
from .typing import NodePredicates
from ..graph import BELGraph
from ...dsl import BaseEntity
from ...typing import Strings

__all__ = [
    "filter_nodes",
    "get_nodes",
    "count_passed_node_filter",
    "summarize_node_filter",
    "get_nodes_by_function",
    "get_nodes_by_namespace",
]


def filter_nodes(graph: BELGraph, node_predicates: NodePredicates) -> Iterable[BaseEntity]:
    """Apply a set of predicates to the nodes iterator of a BEL graph."""
    concatenated_predicate = concatenate_node_predicates(node_predicates=node_predicates)
    for node in graph:
        if concatenated_predicate(graph, node):
            yield node


def get_nodes(graph: BELGraph, node_predicates: NodePredicates) -> Set[BaseEntity]:
    """Get the set of all nodes that pass the predicates."""
    return set(filter_nodes(graph, node_predicates=node_predicates))


def count_passed_node_filter(graph: BELGraph, node_predicates: NodePredicates) -> int:
    """Count how many nodes pass a given set of node predicates."""
    return sum(1 for _ in filter_nodes(graph, node_predicates=node_predicates))


def summarize_node_filter(graph: BELGraph, node_filters: NodePredicates) -> None:
    """Print a summary of the number of nodes passing a given set of filters.

    :param graph: A BEL graph
    :param node_filters: A node filter or list/tuple of node filters
    """
    passed = count_passed_node_filter(graph, node_filters)
    print("{}/{} nodes passed".format(passed, graph.number_of_nodes()))


def get_nodes_by_function(graph: BELGraph, func: Strings) -> Set[BaseEntity]:
    """Get all nodes with the given function(s)."""
    return get_nodes(graph, function_inclusion_filter_builder(func))


def get_nodes_by_namespace(graph, namespaces: Strings) -> Set[BaseEntity]:
    """Get all nodes identified by the given namespace(s)."""
    return get_nodes(graph, namespace_inclusion_builder(namespaces))



# -*- coding: utf-8 -*-

"""Parsing, validation, compilation, and data exchange of Biological Expression Language (BEL)."""

from .canonicalize import (
    edge_to_bel,
    to_bel_script,
    to_bel_script_gz,
    to_bel_script_lines,
)
from .dsl import BaseAbundance, BaseEntity
from .io.api import dump, load
from .io.aws import from_s3, to_s3
from .io.bel_commons_client import from_bel_commons, to_bel_commons
from .io.biodati_client import from_biodati, to_biodati
from .io.cx import (
    from_cx,
    from_cx_file,
    from_cx_gz,
    from_cx_jsons,
    to_cx,
    to_cx_file,
    to_cx_gz,
    to_cx_jsons,
)
from .io.emmaa import from_emmaa
from .io.extras import to_csv, to_gsea, to_sif
from .io.fraunhofer_orientdb import from_fraunhofer_orientdb
from .io.gpickle import (
    from_bytes,
    from_bytes_gz,
    from_pickle,
    from_pickle_gz,
    to_bytes,
    to_bytes_gz,
    to_pickle,
    to_pickle_gz,
)
from .io.graphdati import (
    from_graphdati,
    from_graphdati_file,
    from_graphdati_gz,
    from_graphdati_jsons,
    to_graphdati,
    to_graphdati_file,
    to_graphdati_gz,
    to_graphdati_jsonl,
    to_graphdati_jsonl_gz,
    to_graphdati_jsons,
)
from .io.graphml import to_graphml
from .io.hetionet import (
    from_hetionet_file,
    from_hetionet_gz,
    from_hetionet_json,
    get_hetionet,
)
from .io.hipathia import (
    from_hipathia_dfs,
    from_hipathia_paths,
    to_hipathia,
    to_hipathia_dfs,
)
from .io.indra import (
    from_biopax,
    from_indra_pickle,
    from_indra_statements,
    from_indra_statements_json,
    from_indra_statements_json_file,
    to_indra_statements,
    to_indra_statements_json,
    to_indra_statements_json_file,
)
from .io.jgif import (
    from_cbn_jgif,
    from_cbn_jgif_file,
    from_jgif,
    from_jgif_file,
    from_jgif_gz,
    from_jgif_jsons,
    post_jgif,
    to_jgif,
    to_jgif_file,
    to_jgif_gz,
    to_jgif_jsons,
)
from .io.jupyter import to_jupyter, to_jupyter_str
from .io.lines import from_bel_script, from_bel_script_url
from .io.neo4j import to_neo4j
from .io.nodelink import (
    from_nodelink,
    from_nodelink_file,
    from_nodelink_gz,
    from_nodelink_jsons,
    to_nodelink,
    to_nodelink_file,
    to_nodelink_gz,
    to_nodelink_jsons,
)
from .io.pynpa import to_npa_dfs, to_npa_directory
from .io.sbel import (
    from_sbel,
    from_sbel_file,
    from_sbel_gz,
    to_sbel,
    to_sbel_file,
    to_sbel_gz,
)
from .io.spia import to_spia_dfs, to_spia_excel, to_spia_tsvs
from .io.triples import to_edgelist, to_triples, to_triples_file
from .io.umbrella_nodelink import (
    to_umbrella_nodelink,
    to_umbrella_nodelink_file,
    to_umbrella_nodelink_gz,
)
from .manager import Manager, from_database, to_database
from .parser.parse_bel import parse
from .struct import BELGraph, Pipeline, Query
from .struct.operations import union
from .version import get_version



# -*- coding: utf-8 -*-

"""Constants for PyBEL.

This module maintains the strings used throughout the PyBEL codebase to promote consistency.
"""

from .config import connection


def get_cache_connection() -> str:
    """Get the preferred RFC-1738 database connection string.

    1. Check the environment variable ``PYBEL_CONNECTION``
    2. Check the ``PYBEL_CONNECTION`` key in the config file ``~/.config/pybel/config.json``. Optionally, this config
       file might be in a different place if the environment variable ``PYBEL_CONFIG_DIRECTORY`` has been set.
    3. Return a default connection string using a SQLite database in the ``~/.pybel``. Optionally, this directory
       might be in a different place if the environment variable ``PYBEL_RESOURCE_DIRECTORY`` has been set.
    """
    return connection


PYBEL_CONTEXT_TAG = "pybel_context"
PYBEL_AUTOEVIDENCE = "Automatically added by PyBEL"

CITATION_TYPE_BOOK = "book"
CITATION_TYPE_PUBMED = "pubmed"
CITATION_TYPE_PMC = "pmc"
CITATION_TYPE_URL = "url"
CITATION_TYPE_DOI = "doi"
CITATION_TYPE_OTHER = "other"
CITATION_TYPES = {
    CITATION_TYPE_BOOK,
    CITATION_TYPE_PUBMED,
    CITATION_TYPE_PMC,
    CITATION_TYPE_URL,
    CITATION_TYPE_DOI,
    CITATION_TYPE_OTHER,
}
CITATION_NORMALIZER = {
    "pubmed central": "pmc",
    "pmid": "pubmed",
    "online resource": "url",
}

NAMESPACE_DOMAIN_BIOPROCESS = "BiologicalProcess"
NAMESPACE_DOMAIN_CHEMICAL = "Chemical"
NAMESPACE_DOMAIN_GENE = "Gene and Gene Products"
NAMESPACE_DOMAIN_OTHER = "Other"
#: The valid namespace types
#: .. seealso:: https://wiki.openbel.org/display/BELNA/Custom+Namespaces
NAMESPACE_DOMAIN_TYPES = {
    NAMESPACE_DOMAIN_BIOPROCESS,
    NAMESPACE_DOMAIN_CHEMICAL,
    NAMESPACE_DOMAIN_GENE,
    NAMESPACE_DOMAIN_OTHER,
}

#: Represents the key for the citation date in a citation dictionary
CITATION_DATE = "date"
#: Represents the key for the citation authors in a citation dictionary
CITATION_AUTHORS = "authors"
#: Represents the key for the citation comment in a citation dictionary
CITATION_JOURNAL = "journal"
#: Represents the key for the optional PyBEL citation volume entry in a citation dictionary
CITATION_VOLUME = "volume"
#: Represents the key for the optional PyBEL citation issue entry in a citation dictionary
CITATION_ISSUE = "issue"
#: Represents the key for the optional PyBEL citation pages entry in a citation dictionary
CITATION_PAGES = "pages"
#: Represents the key for the optional PyBEL citation first author entry in a citation dictionary
CITATION_FIRST_AUTHOR = "first"
#: Represents the key for the optional PyBEL citation last author entry in a citation dictionary
CITATION_LAST_AUTHOR = "last"
#: Represents the type of article (Journal Article, Review, etc.)
CITATION_ARTICLE_TYPE = "article_type"

# Used during BEL parsing

MODIFIER = "modifier"
EFFECT = "effect"
FROM_LOC = "fromLoc"
TO_LOC = "toLoc"

LOCATION = "location"

ACTIVITY = "Activity"
DEGRADATION = "Degradation"
TRANSLOCATION = "Translocation"
CELL_SECRETION = "CellSecretion"
CELL_SURFACE_EXPRESSION = "CellSurfaceExpression"

INTRACELLULAR = "intracellular"
EXTRACELLULAR = "extracellular space"
CELL_SURFACE = "cell surface"

# Internal node data format keys
#: The node data key specifying the node's function (e.g. :data:`GENE`, :data:`MIRNA`, :data:`BIOPROCESS`, etc.)
FUNCTION = "function"
#: The key specifying a concept
CONCEPT = "concept"
#: The key specifying an identifier dictionary's namespace. Used for nodes, activities, and transformations.
NAMESPACE = "namespace"
#: The key specifying an identifier dictionary's name. Used for nodes, activities, and transformations.
NAME = "name"
#: The key specifying an identifier dictionary
IDENTIFIER = "identifier"
#: The key specifying an optional label for the node
LABEL = "label"
#: The key specifying an optional description for the node
DESCRIPTION = "description"
#: The key specifying xrefs
XREFS = "xref"

#: They key representing the nodes that are a member of a composite or complex
MEMBERS = "members"
#: The key representing the nodes appearing in the reactant side of a biochemical reaction
REACTANTS = "reactants"
#: The key representing the nodes appearing in the product side of a biochemical reaction
PRODUCTS = "products"

#: The node data key specifying a fusion dictionary, containing :data:`PARTNER_3P`, :data:`PARTNER_5P`,
# :data:`RANGE_3P`, and :data:`RANGE_5P`
FUSION = "fusion"
#: The key specifying the identifier dictionary of the fusion's 3-Prime partner
PARTNER_3P = "partner_3p"
#: The key specifying the identifier dictionary of the fusion's 5-Prime partner
PARTNER_5P = "partner_5p"
#: The key specifying the range dictionary of the fusion's 3-Prime partner
RANGE_3P = "range_3p"
#: The key specifying the range dictionary of the fusion's 5-Prime partner
RANGE_5P = "range_5p"

FUSION_REFERENCE = "reference"
FUSION_START = "left"
FUSION_STOP = "right"
FUSION_MISSING = "missing"

#: The key specifying the node has a list of associated variants
VARIANTS = "variants"
#: The key representing what kind of variation is being represented
KIND = "kind"
#: The value for :data:`KIND` for an HGVS variant
HGVS = "hgvs"
#: The value for :data:`KIND` for a protein modification
PMOD = "pmod"
#: The value for :data:`KIND` for a gene modification
GMOD = "gmod"
#: The value for :data:`KIND` for a fragment
FRAGMENT = "frag"

#: The allowed values for :data:`KIND`
PYBEL_VARIANT_KINDS = {
    HGVS,
    PMOD,
    GMOD,
    FRAGMENT,
}

#: The group of all BEL-provided keys for node data dictionaries, used for hashing.
PYBEL_NODE_DATA_KEYS = {
    FUNCTION,
    NAMESPACE,
    NAME,
    IDENTIFIER,
    VARIANTS,
    FUSION,
    MEMBERS,
    REACTANTS,
    PRODUCTS,
}

#: Used as a namespace when none is given when lenient parsing mode is turned on. Not recommended!
DIRTY = "dirty"

#: Represents the BEL abundance, abundance()
ABUNDANCE = "Abundance"

#: Represents the BEL abundance, geneAbundance()
#: .. seealso:: http://openbel.org/language/version_2.0/bel_specification_version_2.0.html#Xabundancea
GENE = "Gene"

#: Represents the BEL abundance, rnaAbundance()
RNA = "RNA"

#: Represents the BEL abundance, microRNAAbundance()
MIRNA = "miRNA"

#: Represents the BEL abundance, proteinAbundance()
PROTEIN = "Protein"

#: Represents the BEL function, biologicalProcess()
BIOPROCESS = "BiologicalProcess"

#: Represents the BEL function, pathology()
PATHOLOGY = "Pathology"

#: Represents the BEL function, populationAbundance()
POPULATION = "Population"

#: Represents the BEL abundance, compositeAbundance()
COMPOSITE = "Composite"

#: Represents the BEL abundance, complexAbundance()
COMPLEX = "Complex"

#: Represents the BEL transformation, reaction()
REACTION = "Reaction"

#: A set of all of the valid PyBEL node functions
PYBEL_NODE_FUNCTIONS = {
    ABUNDANCE,
    GENE,
    RNA,
    MIRNA,
    PROTEIN,
    BIOPROCESS,
    PATHOLOGY,
    COMPOSITE,
    COMPLEX,
    REACTION,
    POPULATION,
}

#: The mapping from PyBEL node functions to BEL strings
rev_abundance_labels = {
    ABUNDANCE: "a",
    GENE: "g",
    MIRNA: "m",
    PROTEIN: "p",
    RNA: "r",
    BIOPROCESS: "bp",
    PATHOLOGY: "path",
    COMPLEX: "complex",
    COMPOSITE: "composite",
    POPULATION: "pop",
}

# Internal edge data keys

#: The key for an internal edge data dictionary for the relation string
RELATION = "relation"
#: The key for an internal edge data dictionary for the citation dictionary
CITATION = "citation"
CITATION_DB = NAMESPACE  # for backwards compatibility
CITATION_IDENTIFIER = IDENTIFIER  # for backwards compatibility
#: The key for an internal edge data dictionary for the evidence string
EVIDENCE = "evidence"
#: The key for an internal edge data dictionary for the annotations dictionary
ANNOTATIONS = "annotations"
SOURCE = "source"
SUBJECT = SOURCE  # for backwards compatibility
TARGET = "target"
OBJECT = TARGET  # for backwards compatibility
#: The key for an internal edge data dictionary for the source modifier dictionary
SOURCE_MODIFIER = "source_modifier"
#: The key for an internal edge data dictionary for the target modifier dictionary
TARGET_MODIFIER = "target_modifier"
#: The key or an internal edge data dictionary for the line number
LINE = "line"
#: The key representing the hash of the other
HASH = "hash"

#: The group of all BEL-provided keys for edge data dictionaries, used for hashing.
PYBEL_EDGE_DATA_KEYS = {
    RELATION,
    CITATION,
    EVIDENCE,
    ANNOTATIONS,
    SOURCE_MODIFIER,
    TARGET_MODIFIER,
}

#: The group of all PyBEL-specific keys for edge data dictionaries, not used for hashing.
PYBEL_EDGE_METADATA_KEYS = {
    LINE,
    HASH,
}

#: The group of all PyBEL annotated keys for edge data dictionaries
PYBEL_EDGE_ALL_KEYS = PYBEL_EDGE_DATA_KEYS | PYBEL_EDGE_METADATA_KEYS

#: A BEL relationship
HAS_REACTANT = "hasReactant"
#: A BEL relationship
HAS_PRODUCT = "hasProduct"
#: A BEL relationship
HAS_VARIANT = "hasVariant"
#: A BEL relationship
#: :data:`GENE` to :data:`RNA` is called transcription
TRANSCRIBED_TO = "transcribedTo"
#: A BEL relationship
#: :data:`RNA` to :data:`PROTEIN` is called translation
TRANSLATED_TO = "translatedTo"
#: A BEL relationship
INCREASES = "increases"
#: A BEL relationship
DIRECTLY_INCREASES = "directlyIncreases"
#: A BEL relationship
DECREASES = "decreases"
#: A BEL relationship
DIRECTLY_DECREASES = "directlyDecreases"
#: A BEL relationship
CAUSES_NO_CHANGE = "causesNoChange"
#: A BEL relationship
REGULATES = "regulates"
#: A BEL relationship
DIRECTLY_REGULATES = "directlyRegulates"
#: A BEL relationship
BINDS = "binds"
#: A BEL relationship
CORRELATION = "correlation"
#: A BEL relationship
NO_CORRELATION = "noCorrelation"
#: A BEL relationship
NEGATIVE_CORRELATION = "negativeCorrelation"
#: A BEL relationship
POSITIVE_CORRELATION = "positiveCorrelation"
#: A BEL relationship
ASSOCIATION = "association"
#: A BEL relationship
ORTHOLOGOUS = "orthologous"
#: A BEL relationship
ANALOGOUS_TO = "analogousTo"
#: A BEL relationship
IS_A = "isA"
#: A BEL relationship
RATE_LIMITING_STEP_OF = "rateLimitingStepOf"
#: A BEL relationship
SUBPROCESS_OF = "subProcessOf"
#: A BEL relationship
BIOMARKER_FOR = "biomarkerFor"
#: A BEL relationship
PROGONSTIC_BIOMARKER_FOR = "prognosticBiomarkerFor"
#: A BEL relationship, added by PyBEL
EQUIVALENT_TO = "equivalentTo"
#: A BEL relationship, added by PyBEL
PART_OF = "partOf"

#: A set of all causal relationships that have an increasing effect
CAUSAL_INCREASE_RELATIONS = {INCREASES, DIRECTLY_INCREASES}
#: A set of all causal relationships that have a decreasing effect
CAUSAL_DECREASE_RELATIONS = {DECREASES, DIRECTLY_DECREASES}
#: A set of all causal relationships that have an inderminate polarity
CAUSAL_APOLAR_RELATIONS = {REGULATES, DIRECTLY_REGULATES}
#: A set of direct causal relations
DIRECT_CAUSAL_RELATIONS = {DIRECTLY_DECREASES, DIRECTLY_INCREASES, DIRECTLY_REGULATES}
#: A set of direct causal relations
INDIRECT_CAUSAL_RELATIONS = {DECREASES, INCREASES, REGULATES}
#: A set of causal relationships that are polar
CAUSAL_POLAR_RELATIONS = CAUSAL_INCREASE_RELATIONS | CAUSAL_DECREASE_RELATIONS
#: A set of all causal relationships
CAUSAL_RELATIONS = CAUSAL_INCREASE_RELATIONS | CAUSAL_DECREASE_RELATIONS | CAUSAL_APOLAR_RELATIONS

APOLAR_CORRELATIVE_RELATIONS = {
    CORRELATION,
    NO_CORRELATION,
}

POLAR_CORRELATIVE_RELATIONS = {
    POSITIVE_CORRELATION,
    NEGATIVE_CORRELATION,
}

#: A set of all correlative relationships
CORRELATIVE_RELATIONS = APOLAR_CORRELATIVE_RELATIONS | POLAR_CORRELATIVE_RELATIONS

#: A set of polar relations
POLAR_RELATIONS = CAUSAL_POLAR_RELATIONS | POLAR_CORRELATIVE_RELATIONS

#: A set of all relationships that are inherently directionless, and are therefore added to the graph twice
TWO_WAY_RELATIONS = CORRELATIVE_RELATIONS | {
    ASSOCIATION,
    ORTHOLOGOUS,
    ANALOGOUS_TO,
    EQUIVALENT_TO,
    BINDS,
}

#: A list of relationship types that don't require annotations or evidence
UNQUALIFIED_EDGES = {
    HAS_REACTANT,
    HAS_PRODUCT,
    HAS_VARIANT,
    TRANSCRIBED_TO,
    TRANSLATED_TO,
    IS_A,
    EQUIVALENT_TO,
    PART_OF,
    ORTHOLOGOUS,
}

# BEL Keywords

BEL_KEYWORD_SET = "SET"
BEL_KEYWORD_DOCUMENT = "DOCUMENT"
BEL_KEYWORD_DEFINE = "DEFINE"
BEL_KEYWORD_NAMESPACE = "NAMESPACE"
BEL_KEYWORD_ANNOTATION = "ANNOTATION"
BEL_KEYWORD_AS = "AS"
BEL_KEYWORD_URL = "URL"
BEL_KEYWORD_LIST = "LIST"
BEL_KEYWORD_OWL = "OWL"
BEL_KEYWORD_PATTERN = "PATTERN"

BEL_KEYWORD_UNSET = "UNSET"
BEL_KEYWORD_STATEMENT_GROUP = "STATEMENT_GROUP"
BEL_KEYWORD_CITATION = "Citation"
BEL_KEYWORD_EVIDENCE = "Evidence"
BEL_KEYWORD_SUPPORT = "SupportingText"
BEL_KEYWORD_ALL = "ALL"

BEL_KEYWORD_METADATA_NAME = "Name"
BEL_KEYWORD_METADATA_VERSION = "Version"
BEL_KEYWORD_METADATA_DESCRIPTION = "Description"
BEL_KEYWORD_METADATA_AUTHORS = "Authors"
BEL_KEYWORD_METADATA_CONTACT = "ContactInfo"
BEL_KEYWORD_METADATA_LICENSES = "Licenses"
BEL_KEYWORD_METADATA_COPYRIGHT = "Copyright"
BEL_KEYWORD_METADATA_DISCLAIMER = "Disclaimer"
BEL_KEYWORD_METADATA_PROJECT = "Project"

# Internal metadata representation. See BELGraph documentation, since these are shielded from the user by properties.

#: The key for the document metadata dictionary. Can be accessed by :code:`graph.graph[GRAPH_METADATA]`, or by using
#: the property built in to the :class:`pybel.BELGraph`, :func:`pybel.BELGraph.document`
GRAPH_METADATA = "document_metadata"
GRAPH_NAMESPACE_URL = "namespace_url"
GRAPH_NAMESPACE_PATTERN = "namespace_pattern"
GRAPH_ANNOTATION_URL = "annotation_url"
GRAPH_ANNOTATION_MIRIAM = "annotation_miriam"
GRAPH_ANNOTATION_CURIE = "annotation_curie"
GRAPH_ANNOTATION_PATTERN = "annotation_pattern"
GRAPH_ANNOTATION_LIST = "annotation_list"
GRAPH_WARNINGS = "warnings"
GRAPH_PYBEL_VERSION = "pybel_version"
GRAPH_PATH = "path"

#: The key for the document name. Can be accessed by :code:`graph.document[METADATA_NAME]` or by using the property
#: built into the :class:`pybel.BELGraph` class, :func:`pybel.BELGraph.name`
METADATA_NAME = "name"
#: The key for the document version. Can be accessed by :code:`graph.document[METADATA_VERSION]`
METADATA_VERSION = "version"
#: The key for the document description. Can be accessed by :code:`graph.document[METADATA_DESCRIPTION]`
METADATA_DESCRIPTION = "description"
#: The key for the document authors. Can be accessed by :code:`graph.document[METADATA_NAME]`
METADATA_AUTHORS = "authors"
#: The key for the document contact email. Can be accessed by :code:`graph.document[METADATA_CONTACT]`
METADATA_CONTACT = "contact"
#: The key for the document licenses. Can be accessed by :code:`graph.document[METADATA_LICENSES]`
METADATA_LICENSES = "licenses"
#: The key for the document copyright information. Can be accessed by :code:`graph.document[METADATA_COPYRIGHT]`
METADATA_COPYRIGHT = "copyright"
#: The key for the document disclaimer. Can be accessed by :code:`graph.document[METADATA_DISCLAIMER]`
METADATA_DISCLAIMER = "disclaimer"
#: The key for the document project. Can be accessed by :code:`graph.document[METADATA_PROJECT]`
METADATA_PROJECT = "project"

#: Provides a mapping from BEL language keywords to internal PyBEL strings
DOCUMENT_KEYS = {
    BEL_KEYWORD_METADATA_AUTHORS: METADATA_AUTHORS,
    BEL_KEYWORD_METADATA_CONTACT: METADATA_CONTACT,
    BEL_KEYWORD_METADATA_COPYRIGHT: METADATA_COPYRIGHT,
    BEL_KEYWORD_METADATA_DESCRIPTION: METADATA_DESCRIPTION,
    BEL_KEYWORD_METADATA_DISCLAIMER: METADATA_DISCLAIMER,
    BEL_KEYWORD_METADATA_LICENSES: METADATA_LICENSES,
    BEL_KEYWORD_METADATA_NAME: METADATA_NAME,
    BEL_KEYWORD_METADATA_VERSION: METADATA_VERSION,
    BEL_KEYWORD_METADATA_PROJECT: METADATA_PROJECT,
}

#: The keys to use when inserting a graph to the cache
METADATA_INSERT_KEYS = {
    METADATA_NAME,
    METADATA_VERSION,
    METADATA_DESCRIPTION,
    METADATA_AUTHORS,
    METADATA_CONTACT,
    METADATA_LICENSES,
    METADATA_COPYRIGHT,
    METADATA_DISCLAIMER,
}

#: Provides a mapping from internal PyBEL strings to BEL language keywords. Is the inverse of :data:`DOCUMENT_KEYS`
INVERSE_DOCUMENT_KEYS = {v: k for k, v in DOCUMENT_KEYS.items()}

#: A set representing the required metadata during BEL document parsing
REQUIRED_METADATA = {
    METADATA_NAME,
    METADATA_VERSION,
    METADATA_DESCRIPTION,
    METADATA_AUTHORS,
    METADATA_CONTACT,
}

# Modifier parser constants

#: The key for the starting position of a fragment range
FRAGMENT_START = "start"
#: The key for the stopping position of a fragment range
FRAGMENT_STOP = "stop"
#: The key signifying that there is neither a start nor stop position defined
FRAGMENT_MISSING = "missing"
#: The key for any additional descriptive data about a fragment
FRAGMENT_DESCRIPTION = "description"

#: The order for serializing gene modification data
GMOD_ORDER = [KIND, IDENTIFIER]

#: The key for the reference nucleotide in a gene substitution.
#: Only used during parsing  since this is converted to HGVS.
GSUB_REFERENCE = "reference"
#: The key for the position of a gene substitution.
#: Only used during parsing  since this is converted to HGVS
GSUB_POSITION = "position"
#: The key for the effect of a gene substitution.
#: Only used during parsing since this is converted to HGVS
GSUB_VARIANT = "variant"

#: The key for the protein modification code.
PMOD_CODE = "code"
#: The key for the protein modification position.
PMOD_POSITION = "pos"
#: The order for serializing information about a protein modification
PMOD_ORDER = [KIND, IDENTIFIER, PMOD_CODE, PMOD_POSITION]

#: The key for the reference amino acid in a protein substitution.
#: Only used during parsing since this is concerted to HGVS
PSUB_REFERENCE = "reference"
#: The key for the position of a protein substitution. Only used during parsing since this is converted to HGVS.
PSUB_POSITION = "position"
#: The key for the variant of a protein substitution.Only used during parsing since this is converted to HGVS.
PSUB_VARIANT = "variant"

#: The key for the position at which a protein is truncated
TRUNCATION_POSITION = "position"

#: The mapping from BEL namespace codes to PyBEL internal abundance constants
#: ..seealso:: https://wiki.openbel.org/display/BELNA/Assignment+of+Encoding+%28Allowed+Functions%29+for+BEL+Namespaces
belns_encodings = {
    "G": {GENE},
    "R": {RNA, MIRNA},
    "P": {PROTEIN},
    "M": {MIRNA},
    "A": {ABUNDANCE, RNA, MIRNA, PROTEIN, GENE, COMPLEX},
    "B": {PATHOLOGY, BIOPROCESS},
    "O": {PATHOLOGY},
    "C": {COMPLEX},
}

BELNS_ENCODING_STR = "".join(sorted(belns_encodings))

PYBEL_PUBMED = "29048466"
SET_CITATION_FMT = 'SET Citation = {{"{}", "{}"}}'



# -*- coding: utf-8 -*-

"""Gene Modifications.

PyBEL introduces the gene modification tag, gmod(), to allow for the encoding of epigenetic modifications.
Its syntax follows the same style s the pmod() tags for proteins, and can include the following values:

- M
- Me
- methylation
- A
- Ac
- acetylation

For example, the node :code:`g(HGNC:GSK3B, gmod(M))` is represented with the following:

.. code-block:: python

    from pybel.constants import *

    {
        FUNCTION: GENE,
        NAMESPACE: 'HGNC',
        NAME: 'GSK3B',
        VARIANTS: [
            {
                KIND: GMOD,
                IDENTIFIER: {
                    NAMESPACE: BEL_DEFAULT_NAMESPACE,
                    NAME: 'Me',
                },
            },
        ],
    }

The addition of this function does not preclude the use of all other standard functions in BEL; however, other
compilers probably won't support these standards. If you agree that this is useful, please contribute to discussion
in the OpenBEL community.

.. seealso::

    - PyBEL module :py:func:`pybel.parser.modifiers.get_gene_modification_language`
"""

from pyparsing import Group, MatchFirst, ParserElement, oneOf

from ..utils import nest, one_of_tags
from ...constants import CONCEPT, GMOD, IDENTIFIER, KIND, NAME, NAMESPACE
from ...language import gmod_mappings, gmod_namespace

__all__ = [
    "get_gene_modification_language",
]


def _handle_gmod_default(_, __, tokens):
    e = gmod_mappings[gmod_namespace[tokens[0]]]["xrefs"][0]
    tokens[NAMESPACE] = e.namespace
    tokens[IDENTIFIER] = e.identifier
    tokens[NAME] = e.name
    return tokens


gmod_tag = one_of_tags(tags=["gmod", "geneModification"], canonical_tag=GMOD, name=KIND)
gmod_default_ns = oneOf(list(gmod_namespace)).setParseAction(_handle_gmod_default)


def get_gene_modification_language(
    concept_fqualified: ParserElement,
    concept_qualified: ParserElement,
) -> ParserElement:
    """Build a gene modification parser."""
    concept = MatchFirst(
        [
            concept_fqualified,
            concept_qualified,
            gmod_default_ns,
        ]
    )
    return gmod_tag + nest(Group(concept)(CONCEPT))
